/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Aufbau der Elementsteifigkeitsbeziehung 'BG2DEB'                         */
/*  ================================================                         */
/*                                                                           */
/*  BG2DEB beschreibt ein biegesteifies 2D-Element mit 2 Knoten mit je 2     */
/*  Freiheitsgraden. Es kann elastisch gebettet sein und wird durch 4        */
/*  Elementparameter beschrieben: Biegesteifigkeit, Linienlastintensitaeten  */
/*  an den beiden Knoten, Bettungsziffer.                                    */
/*                                                                           */
/*  Die uebergebene Elementnummer ie ist die Nummer der "Benutzer-           */
/*  Numerierung" (1 ... ne) der Elemente, in km_p stehen ebenfalls die       */
/*  Knotennummern der "Benutzer-Numerierung" (1 ... nk).                     */
/*                                                                           */
/*  Die Felder sind allerdings (wie in C ueblich) mit der unteren Index-     */
/*  grenze 0 definiert, auf km_p [3] [0] steht also z. B. die Knotennummer   */
/*  des ersten zum Element 4 gehoerenden Elementknotens (km_p ist mit ke     */
/*  Spalten vereinbart, was einen Zugriff auf dasselbe Element ueber         */
/*  *(km_p + kemx*3) zulaesst.                                               */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    <math.h>
#include    "femset_f.h"

int bg2deb_f (long ie   , double *xy_p , long   *km_p , double *ep_p ,
              long kemx , long    kx   , long   *kf   , long   *ke   ,
              long kp   , double *em_p , double *eb_p)
{
  long     k1 , k2 ;
  double   ei , x1 , x2 , sl , ff , q1 , q2 , k , kk ;

  *ke = 2 ;                           /*  Knoten pro Element                 */
  *kf = 2 ;                           /*  Freiheitsgrade pro Knoten          */

  k1 = *(km_p + (ie-1)*kemx)     ;    /*  Knotennummern des Elements         */
  k2 = *(km_p + (ie-1)*kemx + 1) ;

  ei = *(ep_p + (ie-1)*kp)     ;      /*  Biegesteifigkeit                   */
  k  = *(ep_p + (ie-1)*kp + 3) ;      /*  Bettungsziffer                     */

  x1 = *(xy_p + (k1-1)*kx)     ;      /*  Knotenkoordinaten                  */
  x2 = *(xy_p + (k2-1)*kx)     ;      

  sl = fabs (x2 - x1) ;
  if (sl < 1.e-8) 
  {
    sprintf (WorkString_f , "Element%6ld hat die Lnge%12.6g" , ie , sl) ;
    infbx2_f ("Element wird nicht verwendet!" , WorkString_f) ;
    return 0 ;
  }

  ff = ei / (sl * sl * sl) ;
  kk = k * sl / 420. ;

  *(em_p     ) = *(em_p + 10) =   12.      * ff + 156.      * kk ;
  *(em_p +  1) =                   6. * sl * ff +  22. * sl * kk ;
  *(em_p +  3) =                   6. * sl * ff -  13. * sl * kk ;
  *(em_p +  2) = *(em_p +  8) =  -12.      * ff +  54.      * kk ;
  *(em_p +  4) =                   6. * sl * ff +  22. * sl * kk ;
  *(em_p + 12) =                   6. * sl * ff -  13. * sl * kk ;
  *(em_p +  5) = *(em_p + 15) =    4. * sl * sl * (ff + kk) ;
  *(em_p +  6) = *(em_p +  9) =   -6. * sl * ff +  13. * sl * kk ;
  *(em_p +  7) = *(em_p + 13) =    2. * sl * sl * ff - 3. * sl * sl * kk ;
  *(em_p + 11) = *(em_p + 14) =   -6. * sl * ff -  22. * sl * kk ;

  q1 = *(ep_p + (ie-1)*kp + 1) ;                           /*  Linienlasten  */
  q2 = *(ep_p + (ie-1)*kp + 2) ;

  *(eb_p    ) =    (7. * q1 + 3. * q2) * sl / 20.      ;
  *(eb_p + 1) =    (3. * q1 + 2. * q2) * sl * sl / 60. ;
  *(eb_p + 2) =    (3. * q1 + 7. * q2) * sl / 20.      ;
  *(eb_p + 3) =  - (2. * q1 + 3. * q2) * sl * sl / 60. ;

  return 1 ;
}
