/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Aufbau der Elementmatrizen K und M (Steifigkeits- und Massenmatrix)      */
/*  fuer Biegetraegerelement                                                 */
/*  ===================================================================      */
/*                                                                           */
/*  Die uebergebene Elementnummer ie ist die Nummer der "Benutzer-           */
/*  Numerierung" (1 ... ne) der Elemente, in km_p stehen ebenfalls die       */
/*  Knotennummern der "Benutzer-Numerierung" (1 ... nk).                     */
/*                                                                           */
/*  Die Felder sind allerdings (wie in C ueblich) mit der unteren Index-     */
/*  grenze 0 definiert, auf km_p [3] [0] steht also z. B. die Knotennummer   */
/*  des ersten zum Element 4 gehoerenden Elementknotens (km_p ist mit ke     */
/*  Spalten vereinbart, was einen Zugriff auf dasselbe Element ueber         */
/*  *(km_p + kemx*3) zulaesst.                                               */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    <math.h>
#include    "femset_f.h"

int bg2dkm_f (long ie   , double *xy_p , long   *km_p , double *ep_p ,
              long kemx , long    kx   , long   *kf   , long   *ke   ,
              long kp   , double *ek_p , double *em_p)
{
  long     k1 , k2 ;
  double   ei , rhoA , x1 , x2 , sl , ff ;

  *ke = 2 ;                           /*  Knoten pro Element                 */
  *kf = 2 ;                           /*  Freiheitsgrade pro Knoten          */

  k1 = *(km_p + (ie-1)*kemx)     ;    /*  Knotennummern des Elements         */
  k2 = *(km_p + (ie-1)*kemx + 1) ;

  ei   = *(ep_p + (ie-1)*kp)     ;    /*  Biegesteifigkeit                   */

  x1 = *(xy_p + (k1-1)*kx)     ;      /*  Knotenkoordinaten                  */
  x2 = *(xy_p + (k2-1)*kx)     ;      

  sl = fabs (x2 - x1) ;
  if (sl < 1.e-8) 
  {
    sprintf (WorkString_f , "Element%6ld hat die Lnge%12.6g" , ie , sl) ;
    infbx2_f ("Element wird nicht verwendet!" , WorkString_f) ;
    return 0 ;
  }

  ff = ei / (sl * sl * sl) ;

  *(ek_p     ) = *(ek_p + 10) =   12.      * ff ;
  *(ek_p +  1) = *(ek_p +  3) =    6. * sl * ff ;
  *(ek_p +  2) = *(ek_p +  8) =  -12.      * ff ;
  *(ek_p +  4) = *(ek_p + 12) =    6. * sl * ff ;
  *(ek_p +  5) = *(ek_p + 15) =    4. * sl * sl * ff ;
  *(ek_p +  6) = *(ek_p +  9) =   -6. * sl * ff ;
  *(ek_p +  7) = *(ek_p + 13) =    2. * sl * sl * ff ;
  *(ek_p + 11) = *(ek_p + 14) =   -6. * sl * ff ;

  rhoA = *(ep_p + (ie-1)*kp + 1) ;    /*  Massebelegung                      */
  ff   = rhoA * sl / 420. ;

  *(em_p     ) = *(em_p + 10) =  156.      * ff ;
  *(em_p +  1) = *(em_p +  4) =   22. * sl * ff ;
  *(em_p +  2) = *(em_p +  8) =   54.      * ff ;
  *(em_p +  3) = *(em_p + 12) =  -13. * sl * ff ;
  *(em_p +  5) = *(em_p + 15) =    4. * sl * sl * ff ;
  *(em_p +  6) = *(em_p +  9) =   13. * sl * ff ;
  *(em_p +  7) = *(em_p + 13) =   -3. * sl * sl * ff ;
  *(em_p + 11) = *(em_p + 14) =  -22. * sl * ff ;

  return 1 ;
}
