/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Einbau verhinderter Verschiebungen, die nicht mit einer                  */
/*  Koordinatenrichtung uebereinstimmen (nur 2D-Probleme!)                   */
/*  =======================================================                  */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    <math.h>
#include    "femset_f.h"

void boubet_f (double *a_p  , double *b_p , long ib ,
               long    nk   , long    kf  ,
               long   *la_p , double *aa_p)
{
  long      i  ,  j ,  k , jd , ii   , jj , n ;
  double    ox , sb , cb , beta ;

  n  = nk * kf        ;

  ox = getnox_f (a_p , ib , n) ;
               /*  ... ist ein Wert in der Groessenordnung der Original-
                       Hauptdiagonalelemente, der fuer das Nullsetzen der
                       Verschiebungen als Hauptdiagonalelement benutzt wird  */

  for (i = 0 ; i < nk ; i++)
  {
    if (*(la_p+i) == 1)
    {
      beta = *(aa_p+i) * pi_f /180. ;
      sb   = sin (beta)           ;
      cb   = cos (beta)           ;
      jd   = i * kf * ib          ;    /* Position des Hauptdiagonalelements */
      k    = jd - ib + 1          ;    /* ... und des Elements darueber      */

      for (j = 2 ; j <= ib ; j++)      /* Schleife fuer die Elemente         */
      {                                /* ueber dem Hauptdiagonalelement     */
        if (k >= 0)
        {
          if (j < ib) 
          {
            *(a_p+k)   = *(a_p+k) * cb + *(a_p+k+1) * sb ;
            *(a_p+k+1) = 0. ;
          }
          else
            *(a_p+k)   = *(a_p+k) * cb ;
        }
        
        k = k - ib + 1 ;
      }

      jj = jd + 2      ;
      ii = jd + ib + 1 ;
      k  =  i * kf     ;            /*  ... ist die Zeilennummer (0, 1, ...) */

      for (j = 3 ; j <= ib ; j++ , jj++ , ii++)     /* Elemente in der Zeile */
        if (k + j - 1 < n) 
        {
          *(a_p+jj) = *(a_p+jj) * cb + *(a_p+ii) * sb ;
          *(a_p+ii) = 0. ;
        }

      *(a_p+jd)    = *(a_p+jd) * cb * cb + *(a_p+jd+1 ) * sb * cb * 2.
                                         + *(a_p+jd+ib) * sb * sb ;
      *(a_p+jd+ib) = ox ;
      *(a_p+jd+1)  = 0. ;
      *(b_p+k)     = *(b_p+k) * cb + *(b_p+k+1) * sb ;
      *(b_p+k+1)   = 0. ;
    }
  }
}
