/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Einbau verhinderter und vorgeschriebener Verschiebungen                  */
/*  =======================================================                  */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    <math.h>
#include    "femset_f.h"

void predis_f (double *a_p , double *b_p , long ib ,
               long    ifg , double  pu  , long n  , double ox) ;

void boucon_f (double *a_p  , double *b_p , long ib ,
               long   *kr_p , long    nk  , long kf ,
               long   *li_p , double *pu_p)
{
  long      i  , j , k , ifg , n ;
  double    ox ;

  n  = nk * kf ;

  ox = getnox_f (a_p , ib , n) ;
               /*  ... ist ein Wert in der Groessenordnung der Original-
                       Hauptdiagonalelemente, der fuer das Nullsetzen der
                       Verschiebungen als Hauptdiagonalelement benutzt wird  */

  for (i = 0 ; i < nk ; i++)      /*  Schleife ueber alle Knoten             */
  {
    for (k = 0 ; k < kf ; k++)    /*  Schleife ueber Knoten-Freiheitsgrade   */
    {
      ifg = i * kf + k ;          /*  Freiheitsgrad-POSITION ...             */
      if (*(pu_p+ifg) != 0.)
      {                           /*  ... mit vorgeschriebener Verschiebung  */

        if (*(li_p+ifg) > ifg)    /*  Vorwaertsverweis (ident. Versch.), es  */
                                  /*  wird das "Verweisziel" behandelt       */

          predis_f (a_p , b_p , ib , *(li_p+ifg) , *(pu_p+ifg) , n , ox) ;
        else
          predis_f (a_p , b_p , ib , ifg , *(pu_p+ifg) , n , ox) ;
      }
    }                    /*  Ende Schleife "Vorgeschriebene Verschiebungen"  */
                         /*  fuer Knotenfreiheitsgrade                       */
    j  = *(kr_p+i) ;     /*  "Behinderungs-Indikator"                        */

    while (j > 0)
    {
      k = j - (j / 10) * 10 ;       /*  Freiheitsgradnummer (behindert)      */
      j = (j - k) / 10      ;       /*  k = 1 ... kf  (externe Numerierung)  */

      if (k <= kf)
      {
        ifg = i * kf + k - 1 ;    /*  Behind. FG (interne Numerierung)       */

        if (*(li_p+ifg) > ifg)    /*  Vorwaertsverweis (ident. Versch.), es  */
                                  /*  wird das "Verweisziel" behandelt       */

          predis_f (a_p , b_p , ib , *(li_p+ifg) , 0. , n , ox) ;
        else
          predis_f (a_p , b_p , ib , ifg , 0. , n , ox) ;
      }
    }
  }

  /*  Nullsetzen der "verweisenden" Freiheitsgrade:                          */
  for (i = 0 ; i < n ; i++)
    if (*(li_p+i) > i)
      predis_f (a_p , b_p , ib , i , 0. , n , ox) ;

}

/*  Vorgeschriebene Verschiebung fuer Freiheitsgrad ifg:                     */

void predis_f (double *a_p , double *b_p , long ib ,
               long    ifg , double  pu  , long n  , double ox)
{
  long idia , jj , jb , kb , ii ;

  idia        = ib * ifg ;          /*  Position des Hauptdiagonalelements   */
  jj          = idia     ;
  *(a_p+idia) = ox       ;          /*  ... erhaelt den Standardwert ox      */
  jb          = ifg      ;          /*  "Aufwaertszaehler"                   */
  kb          = ifg      ;          /*  "Abwaertszaehler"                    */

  for (ii = idia+1 ; ii < idia+ib ; ii++)        /*  Schleife ueber den      */
  {                                              /*  ... "Rest der Zeile"    */
     kb++ ;
     if (kb < n) *(b_p+kb) -= *(a_p+ii) * pu ;
                         /* ... manipuliert "unteren Teil der rechten Seite" */

     *(a_p+ii) = 0. ;                    /* ... erzeugt in "Zeile idia" Null */

     jj = jj - ib + 1 ;                  /* a_p-Positionen der "idia-Spalte" */
     jb-- ;
     if (jb >= 0) *(b_p+jb) -= *(a_p+jj) * pu ;
                          /* ... manipuliert "oberen Teil der rechten Seite" */

     if (jj >= 0) *(a_p+jj) = 0. ;         /* ... erzeugt 0 in "Spalte idia" */
  }

  *(b_p+ifg) = pu * ox ;           /*  Rechte Seite erhaelt das ...          */
                                   /*  "ox-fache" der vorgeschr. Verschiebg. */
}
