/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Einbau verhinderter Verschiebungen (Eigenschwingungen)                   */
/*  ======================================================                   */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    <math.h>
#include    "femset_f.h"

void disp0_f (double *a_p , double *b_p , long ib ,
              long    ifg , double ox) ;

void boueig_f (double *a_p  , double *b_p , long ib ,
               long   *kr_p , long    nk  , long kf)
{
  long      i  , j , k , ifg , n ;
  double    ox ;

  n  = nk * kf ;

  ox = getnox_f (a_p , ib , n) ;
               /*  ... ist ein Wert in der Groessenordnung der Original-
                       Hauptdiagonalelemente, der fuer das Nullsetzen der
                       Verschiebungen als Hauptdiagonalelement benutzt wird  */

  for (i = 0 ; i < nk ; i++)      /*  Schleife ueber alle Knoten             */
  {
    j  = *(kr_p+i) ;              /*  "Behinderungs-Indikator"               */

    while (j > 0)
    {
      k = j - (j / 10) * 10 ;       /*  Freiheitsgradnummer (behindert)      */
      j = (j - k) / 10      ;       /*  k = 1 ... kf  (externe Numerierung)  */

      if (k <= kf)
      {
        ifg = i * kf + k - 1 ;    /*  Behind. FG (interne Numerierung)       */
        disp0_f (a_p , b_p , ib , ifg , ox) ;
      }
    }
  }
}

/*  Verhinderte Verschiebung fuer Freiheitsgrad ifg:                         */

void disp0_f (double *a_p , double *b_p , long ib ,
              long    ifg , double ox)
{
  long idia , jj , ii ;

  idia        = ib * ifg ;          /*  Position des Hauptdiagonalelements   */
  jj          = idia     ;
  *(a_p+idia) = ox       ;          /*  ... erhaelt den Standardwert ox      */
  *(b_p+idia) = 0.       ;          /*  ... erhaelt eine Null                */

  for (ii = idia+1 ; ii < idia+ib ; ii++)        /*  Schleife ueber den      */
  {                                              /*  ... "Rest der Zeile"    */
     *(a_p+ii) = 0. ;                    /* ... erzeugt in "Zeile idia" Null */
     *(b_p+ii) = 0. ;                    /* ... erzeugt in "Zeile idia" Null */

     jj = jj - ib + 1 ;                  /* a_p-Positionen der "idia-Spalte" */

     if (jj >= 0)
	 {
		 *(a_p+jj) = 0. ;                  /* ... erzeugt 0 in "Spalte idia" */
		 *(b_p+jj) = 0. ;                  /* ... erzeugt 0 in "Spalte idia" */
	 }
  }
}
