/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Bandweite und Anzahl der Gleichungen fuer die Systemsteifigkeits-        */
/*  beziehung ermitteln                                                      */
/*  =================================================================        */
/*                                                                           */
/*  Eingabe:   km_p  -  Koinzidenzmatrix                                     */
/*             et_p  -  Element-Typen                                        */
/*             ne    -  Elementanzahl                                        */
/*             ke    -  Maximale Knotenanzahl pro Element                    */
/*             kf    -  Maximale Anzahl der Freiheitsgrade pro Knoten        */
/*             nk    -  Knotenanzahl                                         */
/*                                                                           */
/*  Return-Wert:  Halbe Bandweite der Systemsteifigkeitsmatrix               */
/*                                                                           */
/*  Die Anzahl der Gleichingen (Zeilen- bzw. Spaltenanzahl der System-       */
/*  steifigkeitsmatrix) wird auf der Adresse von n_p abgeliefert.            */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    <stdio.h>
#include    <stdlib.h>
#include    "femset_f.h"

long dimeig_f (long *km_p , long *et_p , 
               long  ne   , long  ke   , long  kf   , long nk , long *n_p)
{
  long i , i1 , i2 , id , inr1 , inr2 , kes , ib ;

  ib = 0 ;

  for (i = 0 ; i < ne ; i++)              /*  Schleife ueber alle Elemente   */
  {
    kes = *(et_p + i*5) == 0 ? ke : *(et_p + i*5 + 1) ;  /* Knoten pro Elem. */

    for (i1 = 0 ; i1 < kes - 1 ; i1++)      /* ... bis zum vorletzten El.-Kn.*/
      for (i2 = i1 + 1 ; i2 < kes ; i2++)   /* ... bis zum letzten El.-Kn.   */
      {
        inr1 = *(km_p + i*ke + i1) ;        /* ... zunaechst die Nummern der */
        inr2 = *(km_p + i*ke + i2) ;        /*     Originalknoten            */
        id   = labs (inr1 - inr2)   ;
        if (id > ib) ib = id       ;
      }
  }

  *n_p = nk * kf       ;         /*  Anzahl der Gleichungen                  */
  return (ib + 1) * kf ;         /*  Bandweite der Systemsteifigkeitsmatrix  */
}
