/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Bandweite und Anzahl der Gleichungen fuer die Systemsteifigkeits-        */
/*  beziehung ermitteln                                                      */
/*  =================================================================        */
/*                                                                           */
/*  Eingabe:   km_p  -  Koinzidenzmatrix                                     */
/*             et_p  -  Element-Typen                                        */
/*             li_p  -  Identische Verschbungen                              */
/*             ne    -  Elementanzahl                                        */
/*             ke    -  Maximale Knotenanzahl pro Element                    */
/*             kf    -  Maximale Anzahl der Freiheitsgrade pro Knoten        */
/*             nk    -  Knotenanzahl                                         */
/*                                                                           */
/*  Return-Wert:  Halbe Bandweite der Systemsteifigkeitsmatrix               */
/*                                                                           */
/*  Die Anzahl der Gleichingen (Zeilen- bzw. Spaltenanzahl der System-       */
/*  steifigkeitsmatrix) wird auf der Adresse von n_p abgeliefert.            */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    <stdio.h>
#include    <stdlib.h>
#include    "femset_f.h"

long dimsys_f (long *km_p , long *et_p , long *li_p ,
               long  ne   , long  ke   , long  kf   , long nk , long *n_p)
{
  long i , i1 , i2 , id , inr1  , inr2  , kes ,
       j , j1 , j2 , k1 , k2  , iali1 , iali2 , k   , ib ;

  ib = 0 ;

  for (i = 0 ; i < ne ; i++)              /*  Schleife ueber alle Elemente   */
  {
    kes = *(et_p + i*5) == 0 ? ke : *(et_p + i*5 + 1) ;  /* Knoten pro Elem. */

    for (i1 = 0 ; i1 < kes - 1 ; i1++)      /* ... bis zum vorletzten El.-Kn.*/
      for (i2 = i1 + 1 ; i2 < kes ; i2++)   /* ... bis zum letzten El.-Kn.   */
      {
        inr1 = *(km_p + i*ke + i1) ;        /* ... zunaechst die Nummern der */
        inr2 = *(km_p + i*ke + i2) ;        /*     Originalknoten            */
        id   = labs (inr1 - inr2)   ;
        if (id > ib) ib = id       ;

        /* ... und das waere bereits der gesamte Algorithmus, wenn nicht
               die Moeglichkeit des Verweises ("identische Verschiebungen")
               noch unguenstigere Bandweiten erzeugen koennte. Fuer
               jeden Freiheitsgrad ist ein Verweis ("Alias") auf einen
               Freiheitsgrad eines beliebigen anderen Knotens moeglich. Es
               muessen alle moeglichen Kombinationen der beiden "Originale"
               und aller "Aliasse" untersucht werden:                        */

        j1 = (inr1 - 1) * kf ;
        j2 =  inr1 * kf - 1  ;        /*  ... sind die Grenzpositionen der   */
        k1 = (inr2 - 1) * kf ;        /*      zu beiden Knoten gehoerenden   */
        k2 =  inr2 * kf - 1  ;        /*      Freiheitsgrade im Feld li_ptr  */

        for (j = j1 ; j <= j2 ; j++)
        {
          if (*(li_p + j) > j)        /*  ... gibt es einen Verweis auf      */
          {                           /*      einen anderen Freiheitsgrad    */
            iali1 = *(li_p + j) / kf + 1  ;    /*  ... ist die KNOTEN-Nummer */
            id = labs (iali1 - inr1)       ;   /*  Vergleich mit Original-1  */
            if (id > ib) ib = id          ;
            id = labs (iali1 - inr2)       ;   /*  Vergleich mit Original-2  */
            if (id > ib) ib = id          ;
            for (k = k1 ; k <= k2 ; k++)
              if (*(li_p + k) > k)        /*  ... gibt es einen Verweis auf  */
              {                           /*      Knoten 2                   */
                iali2 = *(li_p + k) / kf + 1  ;
                id = labs (iali1 - iali2)      ;  /* Vergleich der "Aliasse" */
                if (id > ib) ib = id          ;
              }
          }
        }
        for (k = k1 ; k <= k2 ; k++)
          if (*(li_p + k) > k)            /*  ... gibt es einen Verweis auf  */
          {                               /*      Knoten 2                   */
            iali2 = *(li_p + k) / kf + 1  ;
            id = labs (iali2 - inr1)      ;       /* Vergleich "Alias 2"     */
            if (id > ib) ib = id          ;       /* mit Original 1          */
            id = labs (iali2 - inr2)      ;       /* und mit Original 2      */
            if (id > ib) ib = id          ;
          }
      }
  }

  *n_p = nk * kf       ;         /*  Anzahl der Gleichungen                  */
  return (ib + 1) * kf ;         /*  Bandweite der Systemsteifigkeitsmatrix  */
}
