/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Elementtyp aus Element-Charakteristik "vermuten"                         */
/*  ================================================                         */
/*                                                                           */
/*  Der Elementtyp ist in der Matrix et_p_f verzeichnet. Nur dann, wenn      */
/*  dort eine 0 eingetragen ist (kann beim Lesen eines Files einer aelteren  */
/*  FEMSET-Version passieren), sollte die Funktion elmtyp_f befragt werden.  */
/*                                                                           */
/*  Eingabe:   kx  -  "Dimension" des Elements (ein-, zwei- oder dreidimen-  */
/*                    sionales Element                                       */
/*             kf  -  Freiheitsgrade pro Knoten                              */
/*             ke  -  Knoten pro Element                                     */
/*             kp  -  Anzahl der vorzugebenden Elementparameter              */
/*                                                                           */
/*  Return-Wert ist der Elementtyp oder 0, wenn der Elementtyp aus den       */
/*              Angaben nicht ermittelt werden kann.                         */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    "femset_f.h"

long elmtyp_f (long kx , long kf , long ke , long kp)
{
  if      (kx == 2 && kf == 2 && ke == 2 && kp == 1) return FSET_TRUSS2D_f   ;
  else if (kx == 2 && kf == 2 && ke == 2 && kp == 2) return FSET_TRUSS2DT_f  ;
  else if (kx == 1 && kf == 2 && ke == 2 && kp == 3) return FSET_BEND2D_f    ;
  else if (kx == 1 && kf == 2 && ke == 2 && kp == 4) return FSET_BD2DEB_f    ;
  else if (kx == 1 && kf == 2 && ke == 2 && kp == 2) return FSET_BD2DKM_f    ;
  else if (kx == 2 && kf == 3 && ke == 2 && kp == 4) return FSET_BEAM2D_f    ;
  else if (kx == 2 && kf == 3 && ke == 2 && kp == 3) return FSET_BM2DKM_f    ;
  else if (kx == 3 && kf == 3 && ke == 2 && kp == 1) return FSET_TRUSS3D_f   ;
  else if (kx == 3 && kf == 6 && ke == 2 && kp == 6) return FSET_BEAM3D_f    ;
  else if (kx == 2 && kf == 2 && ke == 3 && kp == 3) return FSET_PLANE3_6_f  ;
  else if (kx == 2 && kf == 2 && ke == 4 && kp == 3) return FSET_PLANE4_8_f  ;
  else if (kx == 2 && kf == 2 && ke == 8 && kp == 3) return FSET_PLANE4_16_f ;
  else if (kx == 2 && kf == 1 && ke == 6 && kp == 0) return FSET_TORS3_6_f   ;
  else                                               return 0L ;
}
