/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Aufbau der Elementsteifigkeitsbeziehung                                  */
/*  =======================================                                  */
/*                                                                           */
/*  Die Funktion emstif_f zur Definition der Elementsteifigkeitsbeziehung    */
/*  ist in C-FEMSET als "Dummy" definiert und liefert den Return-Wert 0      */
/*  (Misserfolg). Die aufrufende Funktion elemat_f kann also nur ihre        */
/*  (vordefinierten) Elementtypen beruecksichtigen (emstif_f wird ohnehin    */
/*  nur gestartet, wenn kein bereits definierter Elementtyp von elemat_f     */
/*  gefunden wurde).                                                         */
/*                                                                           */
/*  Wenn ein neuer Elementtyp eingebaut werden soll, ist ihm eine bisher     */
/*  in elemat_f nicht verwendete Elementtyp-Nummer zuzuordnen, so dass       */
/*  emstif_f beim Auftauchen dieses Elementtyps gestartet wird. Der          */
/*  Funktion emstif_f werden alle fuer den Aufbau der Elementsteifigkeits-   */
/*  beziehung benoetigten Informationen uebergeben, emstif_f muss die        */
/*  Elementsteifigkeitsbeziehung (und den Return-Wert 1) an das aufrufende   */
/*  Programm abliefern.                                                      */
/*                                                                           */
/*  Eingabe:   ie    ... ist die Nummer des Elements in der (mit 1 begin-    */
/*                       nenden) "Benutzer-Numerierung"                      */
/*             xy_p  ... ist der Pointer auf das Array mit den Knoten-       */
/*                       koordinaten, die kx Koordinaten des Knotens k       */
/*                       (k ist die mit 1 beginnende "Benutzer-Numerierung"  */
/*                       der Knoten) sind ab Position  xy_p + (i-k) * kx     */
/*                       zu finden                                           */
/*             km_p  ... ist der Pointer auf die Koinzidenzmatrix, die fuer  */
/*                       jedes Element die Knotennummern ("Benutzer-         */
/*                       Numerierung") enthaelt, fuer Element ie ab          */
/*                       Position  km_p + (ie-1) * kmex (man verwende        */
/*                       unbedingt den Parameter kmex, weil bei Modellen     */
/*                       mit unterschiedlichen Elementtypen die Knoten-      */
/*                       anzahl von Element zu Element variieren kann)       */
/*             kp_p  ... ist der Pointer auf die Matrix mit den Element-     */
/*                       parametern (pro Element kp Werte), die Parameter    */
/*                       fuer Element ie findet man ab  km_p + (ie-1) * kp.  */
/*             kemx  ... ist die maximale Knotenanzahl fuer ein Element      */
/*                       (wird in emstif_f fuer die Index-Rechnung in        */
/*                       km_p benoetigt)                                     */
/*             kx    ... ist die "Dimension des Problems (1, 2 oder 3),      */
/*                       (wird in emstif_f fuer die Index-Rechnung in        */
/*                       xy_p benoetigt)                                     */
/*             kp    ... ist die Anzahl der Elementparameter,                */
/*                       (wird in emstif_f fuer die Index-Rechnung in        */
/*                       kp_p benoetigt)                                     */
/*             em_p  ... ist der Pointer auf das erste Matrix-Element der    */
/*                       aufzubauenden Elementsteifigkeitsmatrix             */
/*             eb_p  ... ist der Pointer auf das erste Vektor-Element des    */
/*                       aufzubauenden Elementbelastungsvektors              */
/*                                                                           */
/*  Ausgabe:   kf    -   Anzahl der Freiheitsgrade pro Knoten                */
/*             ke    -   Anzahl der Knoten fuer das in emstif_f behandelte   */
/*                       Element                                             */
/*            *em_p  -   Elementsteifigkeitsmatrix zeilenweise (es brauchen  */
/*                       nur die Matrixelemente oberhalb und auf der         */
/*                       Hauptdiagonalen definiert zu werden), Matrix-       */
/*                       element (1,1) als *(em_p), Matrixelement (1,2) als  */
/*                       *(em_p+1) usw., Matrixelement (4,5) zum Beispiel    */
/*                       als *(em_p+kf*ke*3+4)                               */
/*            *eb_p  -   Elementbelastungsvektor, erstes Vektorelement als   */
/*                       *(eb_p), zweites Vektorelement als *(eb_p+1) usw.   */
/*                                                                           */
/*  Die Funktion emstif_f kann (wie beschrieben) mit relativen Pointer-      */
/*  Adressen aber natuerlich auch mit Arrays arbeiten (Beipiel fuer          */
/*  das Arbeiten mit relativen Pointer-Adressen: rahm2d_f.c, Beispiel fuer   */
/*  das Arbeiten mit einem Array fuer die Elementsteifigkeitsmatrix:         */
/*  rahm3d_f.c).                                                             */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    "femset_f.h"

int emstif_f (long ie   , double *xy_p , long   *km_p , double *ep_p ,
              long kemx , long    kx   , long   *kf   , long   *ke   ,
              long kp   , double *em_p , double *eb_p)
{
  return 0 ;
}
