/* *********************************************************** */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                 */
/*                                                             */
/*  Normieren der Spalten einer Matrix                         */
/*  ==================================                         */
/*                                                             */
/*  Die Spalten der Matrix X (m Zeilen, neiwe Spalten)         */
/*  werden normiert (Summe der Quadrate der Elemente           */
/*  gleich 1).                                                 */
/*                                                             */
/*  Eingabe:   x_p   -  Matrix X mit m*neiwe Elementen         */
/*             m     -  Zeilenanzahl der Matrix X              */
/*             neiwe -  Spaltenanzahl der Matrix X             */
/*                                                             */
/*  Ausgabe   *x_p   -  Matrix X mit normierten Spalten        */
/*                                                             */
/*  Autor: J. Dankert                                          */
/* *********************************************************** */

#include  <math.h>
#include    "femset_f.h"

void evnorm_f (double *x_p , long m , long neiwe)
{
  long    i   , j , ij ;
  double  fac ;

  for (j = 0 ; j < neiwe ; j++)
  {
	  fac = 0. ;
      for (i = 0 , ij = j ; i < m ; i++ , ij += neiwe)
		  fac += *(x_p + ij) * *(x_p + ij) ;

	  if (fac > 0.)
	  {
		  fac = sqrt (fac) ;
          for (i = 0 , ij = j ; i < m ; i++ , ij += neiwe)
		      *(x_p + ij) /= fac ;
	  }
  }
}
