/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Aufbau der Elementsteifigkeitsbeziehung 'STAB3D'                         */
/*  ================================================                         */
/*                                                                           */
/*  Die uebergebene Elementnummer ie ist die Nummer der "Benutzer-           */
/*  Numerierung" (1 ... ne) der Elemente, in km_p stehen ebenfalls die       */
/*  Knotennummern der "Benutzer-Numerierung" (1 ... nk).                     */
/*                                                                           */
/*  Die Felder sind allerdings (wie in C ueblich) mit der unteren Index-     */
/*  grenze 0 definiert, auf km_p [3] [0] steht also z. B. die Knotennummer   */
/*  des ersten zum Element 4 gehoerenden Elementknotens (km_p ist mit ke     */
/*  Spalten vereinbart, was einen Zugriff auf dasselbe Element ueber         */
/*  *(km_p + kemx*3) zulaesst.                                               */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    <math.h>
#include    "femset_f.h"

int fach3d_f (long ie   , double *xy_p , long   *km_p , double *ep_p ,
              long kemx , long    kx   , long   *kf   , long   *ke   ,
              long kp   , double *em_p , double *eb_p)
{
  long     k1 , k2 ;
  double   ea , x1 , y1 , z1 , x2 , y2 , z2 , sl , cx , cy , cz   ,
           cxcx    , cxcy    , cxcz    , cycy    , cycz    , czcz ;

  *ke = 2 ;                           /*  Knoten pro Element                 */
  *kf = 3 ;                           /*  Freiheitsgrade pro Knoten          */

  k1 = *(km_p + (ie-1)*kemx)     ;    /*  Knotennummern des Elements         */
  k2 = *(km_p + (ie-1)*kemx + 1) ;

  ea = *(ep_p + (ie-1)*kp)     ;      /*  Dehnsteifigkeit                    */

  x1 = *(xy_p + (k1-1)*kx)     ;      /*  Knotenkoordinaten                  */
  y1 = *(xy_p + (k1-1)*kx + 1) ;
  z1 = *(xy_p + (k1-1)*kx + 2) ;
  x2 = *(xy_p + (k2-1)*kx)     ;      /*  Knotenkoordinaten                  */
  y2 = *(xy_p + (k2-1)*kx + 1) ;
  z2 = *(xy_p + (k2-1)*kx + 2) ;

  sl = sqrt ((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1)
                                   + (z2 - z1) * (z2 - z1))  ;
  if (sl < 1.e-8) 
  {
    sprintf (WorkString_f , "Element%6ld hat die Lnge%12.6g" , ie , sl) ;
    infbx2_f ("Element wird nicht verwendet!" , WorkString_f) ;
    return 0 ;
  }

  cx = (x2 - x1) / sl ;               /*  Richtungskosinusse                 */
  cy = (y2 - y1) / sl ;
  cz = (z2 - z1) / sl ;

  cxcx = cx * cx * ea / sl ;
  cxcy = cx * cy * ea / sl ;
  cxcz = cx * cz * ea / sl ;
  cycy = cy * cy * ea / sl ;
  cycz = cy * cz * ea / sl ;
  czcz = cz * cz * ea / sl ;

  *(em_p     ) = *(em_p + 21) =   cxcx ;
  *(em_p +  1) = *(em_p + 22) =   cxcy ;
  *(em_p +  2) = *(em_p + 23) =   cxcz ;
  *(em_p +  3) =                - cxcx ;
  *(em_p +  4) = *(em_p +  9) = - cxcy ;
  *(em_p +  5) = *(em_p + 15) = - cxcz ;
  *(em_p +  7) = *(em_p + 28) =   cycy ;
  *(em_p +  8) = *(em_p + 29) =   cycz ;
  *(em_p + 10) =                - cycy ;
  *(em_p + 11) = *(em_p + 16) = - cycz ;
  *(em_p + 14) = *(em_p + 35) =   czcz ;
  *(em_p + 17) =                - czcz ;

  return 1 ;
}
