/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Aufbau der Elementsteifigkeitsbeziehung 'STAB2D' mit Temperatur          */
/*  ===============================================================          */
/*                                                                           */
/*  Die uebergebene Elementnummer ie ist die Nummer der "Benutzer-           */
/*  Numerierung" (1 ... ne) der Elemente, in km_p stehen ebenfalls die       */
/*  Knotennummern der "Benutzer-Numerierung" (1 ... nk).                     */
/*                                                                           */
/*  Die Felder sind allerdings (wie in C ueblich) mit der unteren Index-     */
/*  grenze 0 definiert, auf km_p [3] [0] steht also z. B. die Knotennummer   */
/*  des ersten zum Element 4 gehoerenden Elementknotens (km_p ist mit ke     */
/*  Spalten vereinbart, was einen Zugriff auf dasselbe Element ueber         */
/*  *(km_p + kemx*3) zulaesst.                                               */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    <math.h>
#include    "femset_f.h"
#include    "w_fs_res.h"

int fact2d_f (long ie   , double *xy_p , long   *km_p , double *ep_p ,
              long kemx , long    kx   , long   *kf   , long   *ke   ,
              long kp   , double *em_p , double *eb_p)
{
  long     k1 , k2 ;
  double   ea , x1 , y1 , x2 , y2 , sl , ca , sa , adt ,
           cc , ss , sc ;

  *ke = 2 ;                        /*  Knoten pro Element                    */
  *kf = 2 ;                        /*  Freiheitsgrade pro Knoten             */

  k1 = *(km_p + (ie-1)*kemx)     ; /*  Knotennummern des Elements            */
  k2 = *(km_p + (ie-1)*kemx + 1) ;

  ea  = *(ep_p + (ie-1)*kp)     ;   /*  Dehnsteifigkeit                      */
  adt = *(ep_p + (ie-1)*kp+1)   ;   /*  alpha * Delta-t                      */

  x1 = *(xy_p + (k1-1)*kx)     ;   /*  Knotenkoordinaten                     */
  y1 = *(xy_p + (k1-1)*kx + 1) ;
  x2 = *(xy_p + (k2-1)*kx)     ;   /*  Knotenkoordinaten                     */
  y2 = *(xy_p + (k2-1)*kx + 1) ;

  sl = sqrt ((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1))  ;
  if (sl < 1.e-8) 
  {
    sprintf (WorkString_f , "Element%6ld hat die Lnge%12.6g" , ie , sl) ;
    infbx2_f ("Element wird nicht verwendet!" , WorkString_f) ;
    return 0 ;
  }

  ca = (x2 - x1) / sl ;            /*  Kosinus und Sinus des                 */
  sa = (y2 - y1) / sl ;            /*  Anstiegswinkels                       */

  cc = ca * ca * ea / sl ;
  ss = sa * sa * ea / sl ;
  sc = sa * ca * ea / sl ;

  *(em_p    ) = *(em_p + 10) =   cc ;
  *(em_p + 1) = *(em_p + 11) =   sc ;
  *(em_p + 2) =                - cc ;
  *(em_p + 3) = *(em_p +  6) = - sc ;
  *(em_p + 5) = *(em_p + 15) =   ss ;
  *(em_p + 7) =                - ss ;

  /* Element-Belastungsvektor: */
  
  *(eb_p    ) = - ea * adt * ca ;
  *(eb_p + 1) = - ea * adt * sa ;
  *(eb_p + 2) =   ea * adt * ca ;
  *(eb_p + 3) =   ea * adt * sa ;

  return 1 ;
}
