/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Versuch, den Vektor der Freiheitsgradtypen auf                           */
/*  "intelligente" Art zu erzeugen                                           */
/*  ==============================================                           */
/*                                                                           */
/*  Der Vektor der Freiheitsgradtypen ft_p_f enthaelt kf_f int-Werte, die    */
/*  die Freiheitsgrade eines Knotens beschreiben. Folgender Schluessel       */
/*  gilt fuer die bisher realisierten Elemente:                              */
/*                                                                           */
/*                  1  -  Verschiebung in x-Richtung,                        */
/*                  2  -  Verschiebung in y-Richtung,                        */
/*                  3  -  Verschiebung in z-Richtung,                        */
/*                  4  -  Verdrehung um x-Achse,                             */
/*                  5  -  Verdrehung um x-Achse,                             */
/*                  6  -  Verdrehung um x-Achse                              */
/*                                                                           */
/*  (fuer ein 2D-Rahmenelement wuerden die drei Knotenfreiheitsgrade mit     */
/*  1, 2 und 6 beschrieben werden).                                          */
/*                                                                           */
/*  In fgtype_f wird dieser Vektor gefuellt, indem nach dem "komplizier-     */
/*  testen Element" gesucht wird, eine Kompatibilitaetspruefung mit          */
/*  anderen Elementen wird nicht ausgefuehrt (z. B. waeren 2D- und 3D-       */
/*  Rahmenelemente innerhalb eines Berechnungsmodells nicht miteinander      */
/*  kompatibel).                                                             */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    "femset_f.h"
                     
void fgtype_f ()
{
   long      i , eltyp ;
   int       j ;
   
   for (j = 0 ; j < kf_f ; j++) *(ft_p_f + j) = j ;

   for (i = 0 ; i < ne_f ; i++)
   {
     eltyp = gteltp_f (i) ;
     
     if (eltyp == FSET_BEAM3D_f)                  /* Rahmen-3D, das bisher   */
     {                                            /* komplizierteste Element */
       for (j = 0 ; j < kf_f ; j++) 
                *(ft_p_f + j) = j ; 
       
       return ;
     } 
     
     if (eltyp == FSET_BEAM2D_f)                  /* Rahmen-2D, dann darf es */
     {                                            /* kein 3D-Element geben   */
       for (j = 0 ; j < kf_f ; j++)
                *(ft_p_f + j) = j ;
        
       *(ft_p_f + 2) = 6 ; 
       return ;
     }
   }
} 
