/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Lesen aller Felder des Berechnungsmodells vom File                       */
/*  ==================================================                       */
/*                                                                           */
/*  Eingabe: femmod_p  -  FILE-Pointer der zu lesenden Datei                 */
/*                                                                           */
/*  Return-Wert:    1  --->  Erfolg                                          */
/*                  0  --->  Misserfolg                                      */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    "femset_f.h"

int fnwlns_f (FILE * , long)                   ;
int frddbl_f (FILE * , double * , long , long) ;
int frdint_f (FILE * , long   * , long , long) ;

int finmod_f (FILE *femmod_p , long  kx   , long    kf   , long    ke   ,
                               long  kp   , long    ne   , long    nk   ,
              double *xy_p   , long *km_p , long   *et_p , double *ep_p ,
              double *bk_p   , long *kr_p , double *sc_p , long   *la_p ,
              double *aa_p   , long *li_p , double *pu_p)
{
   char     line [80] ;
   long     i , janf  , jend , j , k ;
   double   a ;

   if (! fnwlns_f (femmod_p , 1L)) return 0 ;

   /****************  Knotenkoordinaten:  ********************/

   if (! frddbl_f (femmod_p , xy_p , nk*kx , 2L)) return 0 ;

   /*******  Koinzidenzmatrix, Elementtypen:  ****************/

   /* Etwas komplizierter wegen der Aufwaertskompatibiltaet
      zur "alten" FEMMOD-Struktur:

      ke ist die "Anzahl der Knoten pro Element" (alt) bzw.
      die "Maximale Anzahl der Knoten pro Element" (neu), in
      jedem Fall die Spaltenanzahl der Koinzidenzmatrix.

      Die Koinzidenzmatrix wird zeilenweise vom File gelesen
      (eine Zeile enthaelt die Information ueber ein Element).
      In der neuen Filestruktur sind den Knotennummern der
      Knoten, die zum Element gehoeren, 5 Werte vorangestellt:
      Elementtyp, Anzahl der Knoten, Anzahl der Elementparameter,
      Dimension des Modells, Freiheitsgrade pro Knoten
      (individuelle Element-Charakteristik).
      Dies wird dadurch deutlich, dass der Elementtyp mit Minus-
      zeichen abgespeichert wird. Der erste Wert einer Zeile
      macht also mit dem Vorzeichen deutlich, ob alte oder
      neue Filestruktur vorliegt.

      Wenn die neue Filestruktur vorliegt, werden die ersten
      5 Werte einer Zeile in der Elementtyp-Matrix (Pointer et_p)
      abgelegt (der Elementtyp selbst, der mit Minuszeichen vom
      File kommt, wird in der Elementtyp-Matrix positiv gespeichert),
      bei altem Filetyp enthaelt die Elementtyp-Matrix ungeaendert
      ihre Null-Elemente.

      Bei neuer Filestruktur wird fuer die Koinzidenzmatrix nur
      die tatsaechliche (gerade gelesene) Anzahl von Knoten
      erwartet, bei alter Filestruktur werden ke Knotennummern
      fuer das Element gelesen. Die "Anzahl der Elementparameter"
      dient nur zur Information und hat keinen Einfluss auf die
      Filestruktur (fuer jedes Element sind in *ep_p genau kp
      Parameter gespeichert).                                       */

   for (i = 0 ; i < ne ; i++) 
   {
     if (fscanf (femmod_p , "%ld" , &j) != 1) return 0 ;
     if (j > 0)
     {
       *(km_p + i*ke) =  j ;       /*  "Alte" Filestruktur, erstes  */
       janf           =  1 ;       /*  gelesenes Element gehoert    */
       jend           = ke ;       /*  zur Koinzidenzmatrix         */
     }
     else 
     {                        /*  "Neue" Filestruktur, erstes      */
       *(et_p + i*5) = - j ;       /*  Element kennzeichnet Elementtyp  */
       janf          =   0 ;
       for (k = 1 ; k <= 4 ; k++)
        if (fscanf (femmod_p , "%ld" , (et_p + i*5 + k)) != 1) return 0 ;
       jend          = *(et_p + i*5 + 1) ;
     }

     for (j = janf ; j < jend ; j++)
       if (fscanf (femmod_p , "%ld" , (km_p + i*ke + j)) != 1) return 0 ;
   }

   if (! fnwlns_f (femmod_p , 2L)) return 0 ;

   /****************  Elementparameter:  *********************/

   for (j = 0 ; j < kp ; j++)
     for (i = 0 ; i < ne ; i++)
       if (fscanf (femmod_p , "%lf" , (ep_p + i*kp + j)) != 1) return 0 ;

   if (! fnwlns_f (femmod_p , 2L)) return 0 ;

   /****************  Knotenlasten:  *************************/

   if (! frddbl_f (femmod_p , bk_p , nk*kf , 2L)) return 0 ;

   /****************  Knotenlagerung:  ***********************/

   if (! frdint_f (femmod_p , kr_p , nk , 0L)) return 0 ;
   if (! fnwlns_f (femmod_p , 2L))             return 1 ;

   /****************  Federsteifigkeiten:  *******************/

   if (! frddbl_f (femmod_p , sc_p , nk*kf , 2L)) return 1 ;

   /****************  Lager mit speziellen Richtungen:  ******/

   if (! frdint_f (femmod_p , la_p , nk , 2L)) return 1 ;

   /******  Winkel fuer Lager mit speziellen Richtungen: *****/

   if (! frddbl_f (femmod_p , aa_p , nk*3 , 2L)) return 1 ;

   /****************  Identische Verschiebungen:  ************/

   if (! frdint_f (femmod_p , li_p , nk*kf , 2L)) return 1 ;

   /****************  Vorgeschriebene Verschiebungen:  *******/

   if (! frddbl_f (femmod_p , pu_p , nk*kf , 0L)) return 1 ;

   /****************  Vektor der Freiheitsgradtypen:   *******/
      
   fgtype_f () ;  /* ... versucht, den Vektor auf 
                         "intelligente" Art zu erzeugen      */

   return 1 ;

   sscanf (line , "%lf" , &a) ;
   /******  Weiss der Geier, warum "floating point not loaded" als
       Run-Time-Error kommt, wenn diese bloede (unerreichbare!)
       Anweisung fehlt! Mal sehen, was GNU-cc dazu sagt!        */
   return 1 ; /*  ... um nicht 2 Warnungen zu provozieren     */
}

int fnwlns_f (FILE *file_p , long n)
{
   char     line [80] ;
   long     i ;

   for (i = 1 ; i <= n ; i++)
      if (fgets  (line , 80 , file_p) == NULL) return 0 ;

   return 1 ;
}

int frddbl_f (FILE *file_p , double *dbl_p , long n , long nbl)
{
   long     i ;

   for (i = 0 ; i < n ; i++)
       if (fscanf (file_p , "%lf" , (dbl_p + i)) != 1) return 0 ;

   if (! fnwlns_f (file_p , nbl)) return 0 ;

   return 1 ;
}

int frdint_f (FILE *file_p , long *int_p , long n , long nbl)
{
   long     i ;

   for (i = 0 ; i < n ; i++)
       if (fscanf (file_p , "%ld" , (int_p + i)) != 1) return 0 ;

   if (! fnwlns_f (file_p , nbl)) return 0 ;

   return 1 ;
}
