/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Schreiben des Berechnungsmodells auf File                                */
/*  =========================================                                */
/*                                                                           */
/*  Eingabe: femmod_p  -  FILE-Pointer der zu schreibenden Datei             */
/*           uvuse     =  1  -  Berechnungsergebnisse (Verschiebungen, die   */
/*                              auf uv_p angeliefert werden) sollen mit      */
/*                              ausgegeben werden                            */
/*                        0  -  Ausgabe des Berechnungsmodells (ohne         */
/*                              Ergebnisse)                                  */
/*                                                                           */
/*  Return-Wert:    1  --->  Erfolg                                          */
/*                  0  --->  Misserfolg                                      */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    "femset_f.h"

int foumod_f (FILE *femmod_p , long  kx   , long    kf   , long ke      ,
                               long  kp   , long    ne   , long nk      ,
              double *xy_p   , long *km_p , long   *et_p , double *ep_p ,
              double *bk_p   , long *kr_p , double *sc_p , long   *la_p ,
              double *aa_p   , long *li_p , double *pu_p , double *uv_p ,
              long    uvuse)
{
   long     iug , iog , i , j , jend ;

   if (fputs  (" Element-Charakteristik:\n" , femmod_p) < 0) return 0 ;
   if (fprintf (femmod_p , "%12ld%12ld%12ld%12ld\n" ,
                                     kx , kf , ke , kp) < 0) return 0 ;

   if (fputs  (" Elementanzahl, Knotenanzahl:\n" ,
                                              femmod_p) < 0) return 0 ;
   if (fprintf (femmod_p , "%12ld%12ld\n" , ne , nk) < 0)    return 0 ;

   if (fputs  (" Knotenkoordinaten:\n" , femmod_p) < 0)      return 0 ;
   for (i = 0 ; i < nk ; i++) 
   {
     for (j = 0 ; j < kx ; j++)
       if (fprintf (femmod_p , "%25.15g" , *(xy_p + i*kx + j)) < 0)
                                                             return 0 ;
       if (fprintf (femmod_p , "\n") < 0)                    return 0 ;
   }

   if (fputs  (" Koinzidenzmatrix:\n" , femmod_p) < 0)       return 0 ;

   /* Etwas komplizierter wegen der Aufwaertskompatibiltaet
      zur "alten" FEMMOD-Struktur (genauere Erlaeuterungen im
      Kommentar der Funktion finmod_f), Prinzip bei der
      Abspeicherung: Was nach "alter" Filestruktur im Programm
      gespeichert war (keine Eintragungen in *et_p), wird auch
      nach "alter" im File abgelegt.                           */

   for (i = 0 ; i < ne ; i++) 
   {
     if (*(et_p + i*5) == 0) 
     {
       jend = ke ;
     }
     else 
     {
       jend = *(et_p + i*5 + 1) ;
       if (fprintf (femmod_p , "%12ld" , - *(et_p + i*5)) < 0) return 0 ;
       for (j = 1 ; j <= 4 ; j++)
         if (fprintf (femmod_p , "%12ld" , *(et_p + i*5 + j)) < 0)
                                                               return 0 ;
     }
     for (j = 0 ; j < jend ; j++)
       if (fprintf (femmod_p , "%12ld" , *(km_p + i*ke + j)) < 0)
                                                               return 0 ;
       if (fprintf (femmod_p , "\n") < 0)                      return 0 ;
   }

   if (fputs  (" Elementparameter:\n" , femmod_p) < 0)         return 0 ;
   
   for (j = 0 ; j < kp ; j++) 
   {
     for (i = 0 ; i < ne ; i++)
       if (fprintf (femmod_p , "%25.15g" , *(ep_p + i*kp + j)) < 0)
                                                               return 0 ;
       if (fprintf (femmod_p , "\n") < 0)                      return 0 ;
   }

   if (fputs  (" Knotenlasten:\n" , femmod_p) < 0)             return 0 ;
   
   for (i = 0 ; i < nk ; i++) 
   {
     for (j = 0 ; j < kf ; j++)
       if (fprintf (femmod_p , "%25.15g" , *(bk_p + i*kf + j)) < 0)
                                                               return 0 ;
       if (fprintf (femmod_p , "\n") < 0)                      return 0 ;
   }

   if (fputs  (" Randbedingungen:\n" , femmod_p) < 0)          return 0 ;
   
   for (i = 0 ; i < nk ; i++)
     if (fprintf (femmod_p , "%12ld" , *(kr_p + i)) < 0)       return 0 ;
   
   if (fprintf (femmod_p , "\n") < 0)                          return 0 ;
   if (fputs  (" Federsteifigkeiten:\n" , femmod_p) < 0)       return 0 ;
   
   for (i = 0 ; i < nk ; i++) 
   {
     for (j = 0 ; j < kf ; j++)
       if (fprintf (femmod_p , "%25.15g" , *(sc_p + i*kf + j)) < 0)
                                                               return 0 ;
       if (fprintf (femmod_p , "\n") < 0)                      return 0 ;
   }

   if (fputs  (" Lager mit speziellen Richtungen:\n" , femmod_p) < 0)
                                                               return 0 ;
   for (i = 0 ; i < nk ; i++)
     if (fprintf (femmod_p , "%12ld" , *(la_p + i)) < 0)       return 0 ;
   
   if (fprintf (femmod_p , "\n") < 0)                          return 0 ;
   if (fputs  (" Winkel fuer Lager mit speziellen Richtungen:\n" ,
                                                femmod_p) < 0) return 0 ;
   for (i = 0 ; i < nk*3 ; i++)
     if (fprintf (femmod_p , "%25.15g" , *(aa_p + i)) < 0)     return 0 ;

   if (fprintf (femmod_p , "\n") < 0)                          return 0 ;
   if (fputs  (" Identische Verschiebungen:\n" , femmod_p) < 0)
                                                               return 0 ;
   for (i = 0 ; i < nk*kf ; i++)
     if (fprintf (femmod_p , "%12ld" , *(li_p + i)) < 0)       return 0 ;

   if (fprintf (femmod_p , "\n") < 0)                          return 0 ;
   if (fputs  (" Vorgeschriebene Verschiebungen:\n" , femmod_p) < 0)
                                                               return 0 ;
   for (i = 0 ; i < nk*kf ; i++)
     if (fprintf (femmod_p , "%25.15g" , *(pu_p + i)) < 0)     return 0 ;

   if (fprintf (femmod_p , "\n") < 0)                          return 0 ;

   if (uvuse) 
   {
     if (fputs  (" Ergebnisse (Knotenverschiebungen):\n" , femmod_p) < 0)
                                                               return 0 ;
     iug = 1 ;
     do 
     {
       iog = iug + 2 ;
       if (iog > kf) iog = kf ;

       for (i = iug ; i <= iog ; i++)
         if (fprintf (femmod_p , "      Freiheitsgrad%2ld    " , i) < 0)
                                                               return 0 ;

       if (fprintf (femmod_p , "\n") < 0)                      return 0 ;

       for (j = 0 ; j < nk ; j++) 
       {
         for (i = iug-1 ; i < iog ; i++)
           if (fprintf (femmod_p , "%25.15g" , *(uv_p + j*kf + i)) < 0)
                                                               return 0 ;

         if (fprintf (femmod_p , "\n") < 0)                    return 0 ;
       }                                      
       
       iug = iog + 1 ;
       
     } while (iog < kf) ;
   }

   return 1 ;
}
