/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Einbau verhinderter Verschiebungen (Eigenschwingungen)                   */
/*  ======================================================                   */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    <math.h>
#include    "femset_f.h"

void dellc_f (double *a_p , double *b_p , long n , long ib , long ifg) ;

long lcdeig_f (double *a_p  , double *b_p , long ib ,
               long   *kr_p , long    nk  , long kf)
{
  long      i  , j , k , ifg , n , nend , off , ifgoff ;

  n    = nk * kf ;
  nend = n       ;
  ifgoff = 0 ;

  for (i = 0 ; i < nk ; i++)      /*  Schleife ueber alle Knoten             */
  {
    j   = *(kr_p+i) ;             /*  "Behinderungs-Indikator"               */
	off = 0 ;

    while (j > 0)
    {
      k = j - (j / 10) * 10 ;       /*  Freiheitsgradnummer (behindert)      */
      j = (j - k) / 10      ;       /*  k = 1 ... kf  (externe Numerierung)  */

      if (k <= kf)
      {
        ifg = i * kf + k - 1 - ifgoff ; /*  Behind. FG (interne Numerierung) */
        dellc_f (a_p , b_p , n , ib , ifg) ;
		nend-- ;
		off++ ;
      }
    }
	ifgoff += off ;
  }
   return nend ;
}

/*  Verhinderte Verschiebung fuer Freiheitsgrad ifg:                         */

void dellc_f (double *a_p , double *b_p , long n , long ib , long ifg)
{
  long idia , ibm1 , i , j , jj , ii ;

  idia        = ib * ifg ;          /*  Position des Hauptdiagonalelements   */
  ibm1        = ib - 1   ;

  for (i = 1 , ii = idia - ibm1 ; i < ib ; i++ , ii -= ibm1)
  {                                              
     if (ii > 0)
	 {
        for (j = i , jj = ii ; j < ibm1 ; j++)
		{
			*(a_p + jj  ) = *(a_p + jj + 1) ;
			*(b_p + jj++) = *(b_p + jj + 1) ;
		}

		*(a_p + jj) = 0. ;
		*(b_p + jj) = 0. ;
	 }
  }

  for (i = ifg , ii = idia ; i < n - 1 ; i++ , ii += ib)
  {                                              
      for (j = 0 , jj = ii ; j < ib ; j++)
		{
			*(a_p + jj  ) = *(a_p + jj + ib) ;
			*(b_p + jj++) = *(b_p + jj + ib) ;
		}
  }
}
