/* *********************************************************** */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                 */
/*                                                             */
/*  Matrizenmultiplikation  A * B = C mit der symmetrischen    */
/*  Bandmatrix A und der Rechteckmatrix B                      */
/*  =======================================================    */
/*                                                             */
/*  Von A sind nur die wesentlichen Elemente zeilenweise       */
/*  in einem eindimensionalen Feld a_p gespeichert (pro Zeile  */
/*  ib Elemente, ib ist die Bandweite). Die letzten ib-1       */
/*  Zeilen muessen durch Null-Elemente aufjeweils ib           */
/*  Elemente ergaenzt werden (ein Beispiel fuer die            */
/*  Speicherung solcher Matrizen findet sich bei               */
/*  der Beschreibung des Unterprogramms cholba_f).             */
/*                                                             */
/*  Eingabe:   a_p   -  Bandmatrix, (eindimensionales Feld     */
/*                      mit ma*ib Elementen)                   */
/*             b_p   -  Pointer auf Matrix mit ma Zeilen und   */
/*                      nb Spalten                             */
/*             ma    -  Zeilenanzahl der Matrizen A und B      */
/*             ib    -  Bandweite der Matrix A                 */
/*             nb    -  Spaltenanzahl der Matrix B             */
/*                                                             */
/*  Ausgabe   *c_p   -  Matrix  C = A * B (ma Zeilen, nb       */
/*                      Spalten)                               */
/*                                                             */
/*  Autor: J. Dankert                                          */
/* *********************************************************** */

#include    "femset_f.h"

void matmub_f (double *a_p , double *b_p , double *c_p ,
               long    ma  , long    ib  , long    nb)
{
  long    i , j , k , ij , ki , ibm1 , kcent , kbeg , kend , koff ;
  double  s ;

  ibm1 = ib - 1 ;

  for (i = 0 , ij = 0 ; i < ma ; i++)
  {
	kcent = i*ib ;

	if (i > ma-ib) 
	  kend = kcent + ma - i - 1 ;
	else
	  kend = kcent + ibm1 ;
 
	if (i < ib)
	{
	  kbeg = kcent - ibm1*i ;
	  koff = 0 ;
	}
	else
	{
	  kbeg = kcent - ibm1 * ibm1 ;
	  koff = i - ib + 1 ;
	}

    for (j = 0 ; j < nb ; j++)
    {
      s = 0. ;

      for (k = kbeg , ki = j+koff*nb ; k < kcent ; k += ibm1)
      {
		  s += *(a_p + k) * *(b_p + ki) ;
		  ki += nb ;
	  }

      for (k = kcent ; k <= kend ; k += 1)
      {
		  s += *(a_p + k) * *(b_p + ki) ;
		  ki += nb ;
	  }

      *(c_p + ij++) = s ;
    }
  }
}
