/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Aufbau der Elementsteifigkeitsbeziehung 'RAHMEN2D'                       */
/*  ==================================================                       */
/*                                                                           */
/*  Die uebergebene Elementnummer ie ist die Nummer der "Benutzer-           */
/*  Numerierung" (1 ... ne) der Elemente, in km_p stehen ebenfalls die       */
/*  Knotennummern der "Benutzer-Numerierung" (1 ... nk).                     */
/*                                                                           */
/*  Die Felder sind allerdings (wie in C ueblich) mit der unteren Index-     */
/*  grenze 0 definiert, auf km_p [3] [0] steht also z. B. die Knotennummer   */
/*  des ersten zum Element 4 gehoerenden Elementknotens (km_p ist mit ke     */
/*  Spalten vereinbart, was einen Zugriff auf dasselbe Element ueber         */
/*  *(km_p + kemx*3) zulaesst.                                               */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    <math.h>
#include    "femset_f.h"

int rahm2d_f (long ie   , double *xy_p , long   *km_p , double *ep_p ,
              long kemx , long    kx   , long   *kf   , long   *ke   ,
              long kp   , double *em_p , double *eb_p)
{
  long     k1 , k2 ;
  double   ei , ea , x1 , y1 , x2 , y2 , sl , ca , sa ,
           cc , ss , sc , ff , ai , q1 , q2 , r1 , r2 ;

  *ke = 2 ;                           /*  Knoten pro Element                 */
  *kf = 3 ;                           /*  Freiheitsgrade pro Knoten          */

  k1 = *(km_p + (ie-1)*kemx)     ;    /*  Knotennummern des Elements         */
  k2 = *(km_p + (ie-1)*kemx + 1) ;

  ei = *(ep_p + (ie-1)*kp)     ;      /*  Biegesteifigkeit, Dehnsteifigkeit  */
  ea = *(ep_p + (ie-1)*kp + 1) ;

  x1 = *(xy_p + (k1-1)*kx)     ;      /*  Knotenkoordinaten                  */
  y1 = *(xy_p + (k1-1)*kx + 1) ;
  x2 = *(xy_p + (k2-1)*kx)     ;      /*  Knotenkoordinaten                  */
  y2 = *(xy_p + (k2-1)*kx + 1) ;

  sl = sqrt ((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1))  ;
  if (sl < 1.e-8) 
  {
    sprintf (WorkString_f , "Element%6ld hat die Lnge%12.6g" , ie , sl) ;
    infbx2_f ("Element wird nicht verwendet!" , WorkString_f) ;
    return 0 ;
  }

  ca = (x2 - x1) / sl ;               /*  Kosinus und Sinus des              */
  sa = (y2 - y1) / sl ;               /*  Anstiegswinkels                    */

  cc = ca * ca ;
  ss = sa * sa ;
  sc = sa * ca ;

  ff = ei / (sl * sl * sl) ;
  ai = ea * sl * sl / ei   ;

  *(em_p     ) = *(em_p + 21) =   (ai * cc + 12. * ss) * ff ;
  *(em_p +  1) = *(em_p + 22) =   (ai - 12.) * sc      * ff ;
  *(em_p +  2) =                - 6. * sl * sa         * ff ;
  *(em_p +  7) = *(em_p + 28) =   (ai * ss + 12. * cc) * ff ;
  *(em_p +  8) =                  6. * sl * ca         * ff ;
  *(em_p + 14) = *(em_p + 35) =   4. * sl * sl         * ff ;

  *(em_p + 23) = - *(em_p + 2) ;
  *(em_p + 29) = - *(em_p + 8) ;

  *(em_p +  3) = - *(em_p     )      ;
  *(em_p +  4) = - *(em_p +  1)      ;
  *(em_p +  5) =   *(em_p +  2)      ;
  *(em_p +  9) =   *(em_p +  4)      ;
  *(em_p + 10) = - *(em_p +  7)      ;
  *(em_p + 11) =   *(em_p +  8)      ;
  *(em_p + 15) = - *(em_p +  2)      ;
  *(em_p + 16) = - *(em_p +  8)      ;
  *(em_p + 17) =   *(em_p + 14) * .5 ;

  q1 = *(ep_p + (ie-1)*kp + 2) ;                           /*  Linienlasten  */
  q2 = *(ep_p + (ie-1)*kp + 3) ;

  r1 = (7. * q1 + 3. * q2) * sl / 20. ;
  r2 = (3. * q1 + 7. * q2) * sl / 20. ;

  *(eb_p    ) = - r1 * sa                             ;
  *(eb_p + 1) =   r1 * ca                             ;
  *(eb_p + 2) =   (3. * q1 + 2. * q2) * sl * sl / 60. ;
  *(eb_p + 3) = - r2 * sa                             ;
  *(eb_p + 4) =   r2 * ca                             ;
  *(eb_p + 5) = - (2. * q1 + 3. * q2) * sl * sl / 60. ;

  return 1 ;
}
