/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Aufbau der Elementmatrizen K und M (Steifigkeits- und Massenmatrix)      */
/*  fuer zweidimensionales Rahmenelement                                     */
/*  ===================================================================      */
/*                                                                           */
/*  Die uebergebene Elementnummer ie ist die Nummer der "Benutzer-           */
/*  Numerierung" (1 ... ne) der Elemente, in km_p stehen ebenfalls die       */
/*  Knotennummern der "Benutzer-Numerierung" (1 ... nk).                     */
/*                                                                           */
/*  Die Felder sind allerdings (wie in C ueblich) mit der unteren Index-     */
/*  grenze 0 definiert, auf km_p [3] [0] steht also z. B. die Knotennummer   */
/*  des ersten zum Element 4 gehoerenden Elementknotens (km_p ist mit ke     */
/*  Spalten vereinbart, was einen Zugriff auf dasselbe Element ueber         */
/*  *(km_p + kemx*3) zulaesst.                                               */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    <math.h>
#include    "femset_f.h"

int rm2dkm_f (long ie   , double *xy_p , long   *km_p , double *ep_p ,
              long kemx , long    kx   , long   *kf   , long   *ke   ,
              long kp   , double *ek_p , double *em_p)
{
  long     k1 , k2 ;
  double   ei , ea , rhoA , x1 , y1 , x2 , y2 , sl , ca , sa ,
           cc , ss , sc   , ff , ai ;

  *ke = 2 ;                           /*  Knoten pro Element                 */
  *kf = 3 ;                           /*  Freiheitsgrade pro Knoten          */

  k1 = *(km_p + (ie-1)*kemx)     ;    /*  Knotennummern des Elements         */
  k2 = *(km_p + (ie-1)*kemx + 1) ;

  ei = *(ep_p + (ie-1)*kp)     ;      /*  Biegesteifigkeit, Dehnsteifigkeit  */
  ea = *(ep_p + (ie-1)*kp + 1) ;

  x1 = *(xy_p + (k1-1)*kx)     ;      /*  Knotenkoordinaten                  */
  y1 = *(xy_p + (k1-1)*kx + 1) ;
  x2 = *(xy_p + (k2-1)*kx)     ;      /*  Knotenkoordinaten                  */
  y2 = *(xy_p + (k2-1)*kx + 1) ;

  sl = sqrt ((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1))  ;
  if (sl < 1.e-8) 
  {
    sprintf (WorkString_f , "Element%6ld hat die Lnge%12.6g" , ie , sl) ;
    infbx2_f ("Element wird nicht verwendet!" , WorkString_f) ;
    return 0 ;
  }

  ca = (x2 - x1) / sl ;               /*  Kosinus und Sinus des              */
  sa = (y2 - y1) / sl ;               /*  Anstiegswinkels                    */

  cc = ca * ca ;
  ss = sa * sa ;
  sc = sa * ca ;

  ff = ei / (sl * sl * sl) ;
  ai = ea * sl * sl / ei   ;

  *(ek_p     ) = *(ek_p + 21) =   (ai * cc + 12. * ss) * ff ;
  *(ek_p +  1) = *(ek_p + 22) =   (ai - 12.) * sc      * ff ;
  *(ek_p +  2) =                - 6. * sl * sa         * ff ;
  *(ek_p +  7) = *(ek_p + 28) =   (ai * ss + 12. * cc) * ff ;
  *(ek_p +  8) =                  6. * sl * ca         * ff ;
  *(ek_p + 14) = *(ek_p + 35) =   4. * sl * sl         * ff ;

  *(ek_p + 23) = - *(ek_p + 2) ;
  *(ek_p + 29) = - *(ek_p + 8) ;

  *(ek_p +  3) = - *(ek_p     )      ;
  *(ek_p +  4) = - *(ek_p +  1)      ;
  *(ek_p +  5) =   *(ek_p +  2)      ;
  *(ek_p +  9) =   *(ek_p +  4)      ;
  *(ek_p + 10) = - *(ek_p +  7)      ;
  *(ek_p + 11) =   *(ek_p +  8)      ;
  *(ek_p + 15) = - *(ek_p +  2)      ;
  *(ek_p + 16) = - *(ek_p +  8)      ;
  *(ek_p + 17) =   *(ek_p + 14) * .5 ;

  rhoA = *(ep_p + (ie-1)*kp + 2) ;    /*  Massebelegung                      */
  ff   = rhoA * sl / 420. ;

  *(em_p     ) = *(em_p + 21) =  (140. * cc + 156. * ss) * ff ;
  *(em_p +  3) = *(em_p + 18) =  ( 70. * cc +  54. * ss) * ff ;
  *(em_p +  7) = *(em_p + 28) =  (140. * ss + 156. * cc) * ff ;
  *(em_p +  8) = *(em_p + 13) =    22. * sl * ca         * ff ;
  *(em_p + 10) = *(em_p + 25) =  ( 70. * ss +  54. * cc) * ff ;
  *(em_p + 11) = *(em_p + 31) =   -13. * sl * ca         * ff ;
  *(em_p + 14) = *(em_p + 35) =     4. * sl * sl         * ff ;
  *(em_p + 16) = *(em_p + 26) =    13. * sl * ca         * ff ;
  *(em_p + 17) = *(em_p + 32) =    -3. * sl * sl         * ff ;
  *(em_p + 29) = *(em_p + 34) =   -22. * sl * ca         * ff ;
  *(em_p +  1) = *(em_p +  6) =   -16. * sc              * ff ;
  *(em_p +  9) = *(em_p + 19) =    16. * sc              * ff ;
  *(em_p +  4) = *(em_p + 24) =    16. * sc              * ff ;
  *(em_p + 22) = *(em_p + 27) =   -16. * sc              * ff ;
  *(em_p +  2) = *(em_p + 12) =   -22. * sl * sa         * ff ;
  *(em_p + 23) = *(em_p + 33) =    22. * sl * sa         * ff ;
  *(em_p +  5) = *(em_p + 30) =    13. * sl * sa         * ff ;
  *(em_p + 15) = *(em_p + 20) =   -13. * sl * sa         * ff ;

  return 1 ;
}
