/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Aufbau der Elementsteifigkeitsbeziehung 'SD6'                            */
/*  =============================================                            */
/*                                                                           */
/*  Die uebergebene Elementnummer ie ist die Nummer der "Benutzer-           */
/*  Numerierung" (1 ... ne) der Elemente, in km_p stehen ebenfalls die       */
/*  Knotennummern der "Benutzer-Numerierung" (1 ... nk).                     */
/*                                                                           */
/*  Die Felder sind allerdings (wie in C ueblich) mit der unteren Index-     */
/*  grenze 0 definiert, auf km_p [3] [0] steht also z. B. die Knotennummer   */
/*  des ersten zum Element 4 gehoerenden Elementknotens (km_p ist mit ke     */
/*  Spalten vereinbart, was einen Zugriff auf dasselbe Element ueber         */
/*  *(km_p + kemx*3) zulaesst.                                               */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    <math.h>

int sd6_f (long ie   , double *xy_p , long   *km_p , double *ep_p ,
           long kemx , long    kx   , long   *kf   , long   *ke   ,
           long kp   , double *em_p , double *eb_p)
{
  long     k1 , k2 , k3 ;
  double   t , e , ny , x1 , y1 , x2 , y2 , x3 , y3 , 
	       x21 , x31 , x32 , y21 , y31 , y32 , aa , qns , qkl , div , fak ;

  *ke = 3 ;                        /*  Knoten pro Element                    */
  *kf = 2 ;                        /*  Freiheitsgrade pro Knoten             */

  k1 = *(km_p + (ie-1)*kemx)     ; /*  Knotennummern des Elements            */
  k2 = *(km_p + (ie-1)*kemx + 1) ;
  k3 = *(km_p + (ie-1)*kemx + 2) ;

  t  = *(ep_p + (ie-1)*kp)     ;   /*  Elementparameter                      */
  e  = *(ep_p + (ie-1)*kp + 1) ;
  ny = *(ep_p + (ie-1)*kp + 2) ;

  x1 = *(xy_p + (k1-1)*kx)     ;   /*  Knotenkoordinaten                     */
  y1 = *(xy_p + (k1-1)*kx + 1) ;
  x2 = *(xy_p + (k2-1)*kx)     ;   
  y2 = *(xy_p + (k2-1)*kx + 1) ;
  x3 = *(xy_p + (k3-1)*kx)     ;   
  y3 = *(xy_p + (k3-1)*kx + 1) ;

  x21 = x2 - x1 ;                  /*  Elementabmessungen                    */
  x31 = x3 - x1 ;
  x32 = x3 - x2 ;
  y21 = y2 - y1 ;
  y31 = y3 - y1 ;
  y32 = y3 - y2 ;

  aa  = fabs ((x21 * y31 - x31 * y21) * 0.5)  ;
  qns = (1. - ny) * 0.5 ;
  qkl = qns + ny ;
  div = (1. - ny * ny) * aa * 4. ;

  if (fabs(div) < 1.e-8) return 0 ;
  
  fak = e * t / div ;

  *(em_p     ) = (  qns * x32 * x32 + y32 * y32	     ) * fak ;
  *(em_p +  1) = (- qkl * x32 * y32		             ) * fak ;
  *(em_p +  2) = (- qns * x32 * x31 - y32 * y31	     ) * fak ;
  *(em_p +  3) = (  qns * x32 * y31 + ny  * x31 * y32) * fak ;
  *(em_p +  4) = (  qns * x32 * x21 + y32 * y21	     ) * fak ;
  *(em_p +  5) = (- qns * x32 * y21 - ny  * x21 * y32) * fak ;
  *(em_p +  7) = (  x32 * x32	    + qns * y32 * y32) * fak ;
  *(em_p +  8) = (  ny  * x32 * y31 + qns * x31 * y32) * fak ;
  *(em_p +  9) = (- x32 * x31	    - qns * y32 * y31) * fak ;
  *(em_p + 10) = (- ny  * x32 * y21 - qns * x21 * y32) * fak ;
  *(em_p + 11) = (  x32 * x21	    + qns * y32 * y21) * fak ;
  *(em_p + 14) = (  qns * x31 * x31 + y31 * y31	     ) * fak ;
  *(em_p + 15) = (- qkl * x31 * y31		             ) * fak ;
  *(em_p + 16) = (- qns * x21 * x31 - y21 * y31	     ) * fak ;
  *(em_p + 17) = (  ny  * x21 * y31 + qns * x31 * y21) * fak ;
  *(em_p + 21) = (  x31 * x31	    + qns * y31 * y31) * fak ;
  *(em_p + 22) = (  qns * x21 * y31 + ny  * x31 * y21) * fak ;
  *(em_p + 23) = (- x21 * x31	    - qns * y21 * y31) * fak ;
  *(em_p + 28) = (  qns * x21 * x21 + y21 * y21      ) * fak ;
  *(em_p + 29) = (- qkl * x21 * y21		             ) * fak ;
  *(em_p + 35) = (  x21 * x21	    + qns * y21 * y21) * fak ;

  return 1 ;
}
