/* **************************************************************************/
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                              */
/*                                                                          */
/*  Einbau verhinderter und vorgeschriebener Verschiebungen                 */
/*  ("Skyline-Variante")                                                    */
/*  =======================================================                 */
/*                                                                          */
/*  Autor: J. Dankert                                                       */
/* **************************************************************************/

#include    <math.h>
#include    "femset_f.h"

void presky_f (double **mdiag_pp , long *height_p , double *b_p ,
               long    ifg , double  pu , long n  , double ox) ;

void skybcn_f (double **mdiag_pp , long *height_p , double *b_p , 
               long   *kr_p , long    nk  , long kf ,
               long   *li_p , double *pu_p)
   {
    long      i  , j , k , ifg , n ;
    double    ox ;

    n  = nk * kf ;

    ox = skynox_f (mdiag_pp , n) ;
              /*  ... ist ein Wert in der Groessenordnung der Original-
                      Hauptdiagonalelemente, der fuer das Nullsetzen der
                      Verschiebungen als Hauptdiagonalelement benutzt wird  */

    for (i = 0 ; i < nk ; i++)   /*  Schleife ueber alle Knoten             */
    {
      for (k = 0 ; k < kf ; k++) /*  Schleife ueber Knoten-Freiheitsgrade   */
      {
        ifg = i * kf + k ;       /*  Freiheitsgrad-POSITION ...             */
        if (*(pu_p+ifg) != 0.)
        {                        /*  ... mit vorgeschriebener Verschiebung  */

          if (*(li_p+ifg) > ifg) /*  Vorwaertsverweis (ident. Versch.), es  */
                                 /*  wird das "Verweisziel" behandelt       */

           presky_f (mdiag_pp , height_p , b_p ,
                      *(li_p+ifg) , *(pu_p+ifg) , n , ox) ;
          else
           presky_f (mdiag_pp , height_p , b_p , ifg , *(pu_p+ifg) , n , ox) ;
        }
      }                 /*  Ende Schleife "Vorgeschriebene Verschiebungen"  */
                        /*  fuer Knotenfreiheitsgrade                       */
      j  = *(kr_p+i) ;  /*  "Behinderungs-Indikator"                        */

      while (j > 0)
      {
        k = j - (j / 10) * 10 ;    /*  Freiheitsgradnummer (behindert)      */
        j = (j - k) / 10      ;    /*  k = 1 ... kf  (externe Numerierung)  */

        if (k <= kf)
        {
          ifg = i * kf + k - 1 ; /*  Behind. FG (interne Numerierung)       */

          if (*(li_p+ifg) > ifg) /*  Vorwaertsverweis (ident. Versch.), es  */
                                 /*  wird das "Verweisziel" behandelt       */

            presky_f (mdiag_pp , height_p , b_p , *(li_p+ifg) , 0. , n , ox) ;
          else
            presky_f (mdiag_pp , height_p , b_p , ifg , 0. , n , ox) ;
        }
      }
    }

    /*  Nullsetzen der "verweisenden" Freiheitsgrade: */
    for (i = 0 ; i < n ; i++)
      if (*(li_p+i) > i)
            presky_f (mdiag_pp , height_p , b_p , i , 0. , n , ox) ;

   }

   /*  Vorgeschriebene Verschiebung fuer Freiheitsgrad ifg:                 */
   /*                                                                       */
   /*  Die Spalte ifg wird mit der vorgeschriebenen Verschiebung pu         */
   /*  multipliziert. Die Ergebnisse werden auf die rechte Seite gebracht,  */
   /*  so dass die gesamte rechte Seite betroffen ist (Subtraktion des      */
   /*  Produkts a[i,ifg]*pu von b[i] fuer alle i != ifg, wobei die a[i,ifg] */
   /*  fuer i < ifg aus der ZEILE ifg genommen werden muessen). Die         */
   /*  Elemente a[i,ifg] der Spalte ifg erhalten (bis auf das Haupt-        */
   /*  diagonalelement den Wert Null.                                       */
   /*                                                                       */
   /*  Eine Sonderbehandlung bekommt die Zeile ifg: Alle Elemente ausser    */
   /*  a[ifg,ifg] werden Null gesetzt, auf a[ifg,ifg] wird der Wert ox      */
   /*  gespeichert und auf der rechten Seite wird das Produkt pu*ox auf     */
   /*  b[ifg] abgelegt.                                                     */
   /*                                                                       */
   /*  Der Parameter ifg ist "zero-based", kann also die Werte 0...n-1      */
   /*  haben.                                                               */

   void presky_f (double **mdiag_pp , long *height_p , double *b_p ,
                  long    ifg , double  pu , long n , double ox)
   {
    long ii , k ;

    *(*(mdiag_pp + ifg)) = ox ;    /*  ... erhaelt den Standardwert ox      */
    k = 1 ;

    for (ii = ifg + 1 ; ii < n ; ii++ , k++)    /*  Schleife ueber den      */
    {                                           /*  ... "Rest der Zeile"    */
        if (k <= *(height_p + ii)) 
        {
          *(b_p + ii) -= *(*(mdiag_pp + ii) + k) * pu ;
          *(*(mdiag_pp + ii) + k) = 0. ;
        }  
    }    
    
    k = *(height_p + ifg) ;
    for (ii = 1 ; ii <= k ; ii++)               /*  Schleife ueber den      */
    {                                           /*  ... ifg-"Schornstein"   */
    
        *(b_p + ifg - ii) -= *(*(mdiag_pp + ifg) + ii) * pu ;
        *(*(mdiag_pp + ifg) + ii) = 0. ;
    }    

    *(b_p+ifg) = pu * ox ;        /*  Rechte Seite erhaelt das ...          */
                                  /*  "ox-fache" der vorgeschr. Verschiebg. */
   }
