/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Einbau verhinderter Verschiebungen, die nicht mit einer                  */
/*  Koordinatenrichtung uebereinstimmen (nur 2D-Probleme!), Skyline-Variante */
/*  ======================================================================== */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    <math.h>
#include    "femset_f.h"

void skybet_f (double **mdiag_pp , long *height_p , double *b_p , 
               long    nk   , long    kf  ,
               long   *la_p , double *aa_p)
{
  long      i  ,  j ,  k , ii   , n     , hu    ;
  double    ox , sb , cb , beta , *du_p , *dv_p , *u_p , * v_p ;

  n  = nk * kf        ;

  ox = skynox_f (mdiag_pp , n) ;
               /*  ... ist ein Wert in der Groessenordnung der Original-
                       Hauptdiagonalelemente, der fuer das Nullsetzen der
                       Verschiebungen als Hauptdiagonalelement benutzt wird  */

  for (i = 0 ; i < nk ; i++)
  {
    if (*(la_p+i) == 1)
    {
      beta = *(aa_p+i) * pi_f /180.   ;  
      sb   = sin (beta)               ;
      cb   = cos (beta)               ;
      du_p = *(mdiag_pp + i * kf)     ; /* Position u-Diagonalelement        */
      dv_p = *(mdiag_pp + i * kf + 1) ; /* Position v-Diagonalelement        */
      hu   = *(height_p + i * kf)     ; /* Hoehe des u-Schornsteins          */

      for (j = 1 ; j <= hu ; j++)       /* Schleife fuer die Elemente        */
      {                                 /* ueber dem Hauptdiagonalelement    */
        u_p  = du_p + j     ;
        v_p  = dv_p + j + 1 ;
        *u_p = *u_p * cb + *v_p * sb ;
        *v_p = 0. ;
      }
                                                          
      ii = 2 ;
      for (j = i * kf + 2 ; j < n ; j++)   /* Schleife fuer die Elemente     */
      {                                    /* rechts vom Diagonalelement     */
        if (ii <= *(height_p + j))
        {  
          u_p  = *(mdiag_pp + j) + ii ;
          v_p  = u_p - 1 ;
          *u_p = *u_p * cb + *v_p * sb ;
          *v_p = 0. ;
        }     
        
        ii++ ;
      }
      
      *du_p = *du_p * cb * cb + *(dv_p + 1) * sb * cb * 2. + *dv_p * sb * sb ;
      *dv_p = ox ;
      *(dv_p + 1) = 0. ;
      
      k = i * kf ;
      
      *(b_p + k)     = *(b_p + k) * cb + *(b_p + k + 1) * sb ;
      *(b_p + k + 1) = 0. ;
    }
  }
}
