/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Schornsteinhoehen der Spalten der Systemsteifigkeitsmatrix               */
/*  ==========================================================               */
/*                                                                           */
/*  Eingabe:   km_p  -  Koinzidenzmatrix                                     */
/*             et_p  -  Element-Typen                                        */
/*             li_p  -  Identische Verschbungen                              */
/*             ne    -  Elementanzahl                                        */
/*             kfmx  -  Maximale Anzahl der Freiheitsgrade pro Knoten        */
/*             kemx  -  Maximale Knotenanzahl pro Element                    */
/*             nk    -  Knotenanzahl                                         */
/*             n_p   -  Anzahl der Freiheitsgrade                            */
/*                                                                           */
/*  Die "Schornsteinhoehen" werden auf dem Feld height_p abgeliefert.        */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    "femset_f.h"

void alihgt_f (long *li_p , long is , long js , long *height_p) ;

void skyhgt_f (long *km_p , long *et_p , long *li_p ,
               long  ne   , long  kfmx , long kemx  ,
               long  nk   , long n_p   , long *height_p)
{
    long      i   , j   , ie   , ik   , jk    , iy   , jy  , kf , ke  ,
              iis , jjs , iiis , jjjs , isave ;

    for (i = 0 ; i < n_p ; i++) *(height_p + i) = 0 ;

    for (ie = 0 ; ie < ne ; ie++)       /*  Elementnummern  ie = 0 ... ne-1  */
    {
      ke = *(et_p + ie*5) == 0 ? kemx : *(et_p + ie*5 + 1) ; /* Knt. pro El. */
      kf = kfmx ;

      /*  Speicherziele fuer Systemsteifigkeitsmatrix:                       */

      for (ik = 0 ; ik < ke ; ik++)           /*  Elementknoten werden       */
      {                                       /*  von  0 ... ke-1  gezaehlt  */

        iy = *(km_p + ie*kemx + ik) ;         /*  ... sind die EXTERNEN ...  */

        for (jk = ik ; jk < ke ; jk++)
        {
          jy = *(km_p + ie*kemx + jk) ;       /*  (1 ... nk) KNOTENNUMMERN   */

          iis = (iy - 1) * kfmx     ;         /*  Freiheitsgrad-POSITIONEN   */
          jjs = (jy - 1) * kfmx     ;         /*  des Ziel-Leitelements      */

          if (jy == iy)                       /*  Hauptdiagonaldatenblock    */
          {                                   /*  =======================    */
            iiis = iis ;
            for (i = 0 ; i < kf ; i++ , iiis++ , jjs++)
            {
              for (j = i , jjjs = jjs ; j < kf ; j++ , jjjs++)
              {
                alihgt_f (li_p , iiis , jjjs , height_p) ;
              }
            }    /*  Ende der Schleifen ueber Hauptdiagonaldatenblock        */
          }      /*  Ende der Einspeicherung eines Hauptdiagonaldatenblocks  */

          else if (jy > iy)                   /*  Nebendiagonaldatenblock    */
          {                                   /*  =======================    */
            iiis = iis ;
            for (i = 0 ; i < kf ; i++ , iiis++)
            {
              for (j = 0 , jjjs = jjs ; j < kf ; j++ , jjjs++)
              {
                alihgt_f (li_p , iiis , jjjs , height_p) ;
              }
            }    /*  Ende der Schleifen ueber Nebendiagonaldatenblock        */
          }      /*  Ende der Einspeicherung eines Nebendiagonaldatenblocks  */

          else                                 /*  Transp. Nebendiag.-Block  */
          {                                    /*  ========================  */
            isave = iis   ;                    /*  Zeilen- und Spaltennum-   */
            iis   = jjs   ;                    /*  mern fuer SM werden       */
            jjs   = isave ;                    /*  werden getauscht          */
            iiis  = iis   ;
            for (i = 0 ; i < kf ; i++ , iiis++)
            {
              for (j = 0 , jjjs = jjs ; j < kf ; j++ , jjjs++)
              {
                alihgt_f (li_p , iiis , jjjs , height_p) ;
              }
            }    /*  Ende der Schleifen ueber transp. Nebendiagonaldatenbl.  */
          }      /*  Ende der Einspeicherung des transp. Nebendiag.-Datenbl. */
          
        }        /*  Ende der EM-Spaltenschleife                             */

      }          /*  ... Ende des Einspeichern eines Elements                */
      
    }            /*  ... Ende der Schleife ueber alle Elemente               */
}

/*  Ueberpruefen, ob der Vektor li_p (identische Verschiebungen)
    Alias-Verweise auf andere (hoehere) Freiheitsgrade fuer die
    Freiheitsgrade is und js enthaelt, zurueckgeliefert wird in
    jedem Fall die Position in der Systemsteifigkeitsmatrix oberhalb
    der Hauptdiagonalen, entweder fuer die Original-Freiheitsgrade
    oder unter Beruecksichtigung von "Aliassen":                             */

void alihgt_f (long *li_p , long is , long js , long *height_p)
{
  long  i , j , k ;

  i = (*(li_p + is) > is) ? *(li_p + is) : is ;
  j = (*(li_p + js) > js) ? *(li_p + js) : js ;

  /*  ... und i und j sind die tatsaechlich zu verwendenden
          Ziele fuer das Einspeichern                                        */

  if (i > j)                   /*  ... wuerde Ziel unter H.-Diag. liegen     */
  {
    k = i ;
    i = j ;
    j = k ;                    /*  ... es wurde getauscht                    */
  }      
    
  k = j - i ;
  if (k > *(height_p + j)) *(height_p + j) = k ;
}
