/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Aufbau der Systemsteifigkeitsbeziehung (Skyline-Methode)                 */
/*  ========================================================                 */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    "femset_f.h"
#include    "w_fs_res.h"

void alskys_f (long *li_p , long *is , long *js) ;

int skysys_f (double **mdiag_pp , long *height_p , double *b_p , 
              long    n    , long    ne , long    nk   , long    kx   ,
              long    kfmx , long  kemx , long    kp   , double *xy_p ,
              long   *km_p , long *et_p , double *ep_p , long   *li_p)
{
    double   *em_p , *eb_p ;                /*  Elementsteifigkeitsbeziehung */

    long      i   , j    , ie   , ik  , jk    , iy   , jy  , kf  , ke  ,
              me  , it   , jt   , il  , jl    , suc  , iis , smi , smj ,
              jjs , iiis , jjjs , lls , isave , kfke ;

    me = kfmx * kemx ;         /* Maximale Anzahl der Elementfreiheitsgrade  */

    /***********  Speicherplatz fuer                        ******************/
    /***********  Elementsteifigkeitsbeziehung allokieren:  ******************/

    suc = ((em_p = (double *) calloc ((int) (me * me) ,
                                      sizeof (double))) != NULL) ;
    if (suc) suc = ((eb_p = (double *) calloc ((int) me ,
                                               sizeof (double))) != NULL) ;
    if (! suc) 
    {
      infbox_f (S_SORRY_f , S_NOELEMSPACE_f) ;
      return 0 ;
    }
    
    for (i = 0 ; i < n ; i++)
    {
      darnul_f (*(mdiag_pp + i) , *(height_p + i) + 1) ;
    }
    
    fillin_f (0 , 0L , 0L) ;

    for (ie = 0 ; ie < ne ; ie++)       /*  Elementnummern  ie = 0 ... ne-1  */
    {
      darnul_f (em_p , me * me) ;       /*  Nullsetzen der                   */
      darnul_f (eb_p , me     ) ;       /*  Elementsteifigkeitsbeziehung     */

      ke = kemx ;        /*  Es werden Maximalwerte vermutet, die elemat_f   */
      kf = kfmx ;        /*  modifizieren kann                               */

      elemat_f (ie + 1 , xy_p , km_p , et_p , ep_p , kemx ,
                kx     , &kf  , &ke  , kp   , em_p , eb_p) ;

      kfke = kf * ke ;

      /*  Einspeichern in die Systemsteifigkeitsbeziehung:                   */

      for (ik = 0 ; ik < ke ; ik++)           /*  Elementknoten werden       */
      {                                       /*  von  0 ... ke-1  gezaehlt  */

        iy = *(km_p + ie*kemx + ik) ;         /*  ... sind die EXTERNEN ...  */

        for (jk = ik ; jk < ke ; jk++)
        {
          jy = *(km_p + ie*kemx + jk) ;       /*  (1 ... nk) KNOTENNUMMERN   */

          iis = (iy - 1) * kfmx     ;         /*  Freiheitsgrad-POSITIONEN   */
          jjs = (jy - 1) * kfmx     ;         /*  des Ziel-Leitelements      */

          it = ik * kf              ;         /*  POSITIONEN des Leit-       */
          jt = jk * kf              ;         /*  elements in EM             */

          if (jy == iy)                       /*  Hauptdiagonaldatenblock    */
          {                                   /*  =======================    */
            il   = it  ;
            iiis = iis ;
            for (i = 0 ; i < kf ; i++ , il++ , iiis++ , jjs++)
            {
              for (j = i , jl = il , jjjs = jjs ; j < kf ; j++ , jjjs++)
              { 
                smi = iiis ;
                smj = jjjs ;
                alskys_f (li_p , &smi , &smj) ;
                *(*(mdiag_pp + smj) + smi) += *(em_p + il * kfke + jl++) ;
              }
            }    /*  Ende der Schleifen ueber Hauptdiagonaldatenblock        */
          }      /*  Ende der Einspeicherung eines Hauptdiagonaldatenblocks  */

          else if (jy > iy)                   /*  Nebendiagonaldatenblock    */
          {                                   /*  =======================    */
            il   = it  ;
            iiis = iis ;
            for (i = 0 ; i < kf ; i++ , il++ , iiis++)
            {
              for (j = 0 , jl = jt , jjjs = jjs ; j < kf ; j++ , jjjs++)
              {
                smi = iiis ;
                smj = jjjs ;
                alskys_f (li_p , &smi , &smj) ;
                *(*(mdiag_pp + smj) + smi) += *(em_p + il * kfke + jl++) ;
              }
            }   /*  Ende der Schleifen ueber Nebendiagonaldatenblock        */
          }     /*  Ende der Einspeicherung eines Nebendiagonaldatenblocks  */

          else                                /*  Transp. Nebendiag.-Block  */
          {                                   /*  ========================  */
            jl    = jt    ;
            isave = iis   ;                   /*  Zeilen- und Spaltennum-   */
            iis   = jjs   ;                   /*  mern fuer SM werden       */
            jjs   = isave ;                   /*  werden getauscht          */
            iiis  = iis   ;
            for (i = 0 ; i < kf ; i++ , jl++ , iiis++)
            {
              for (j = 0 , il = it , jjjs = jjs ; j < kf ; j++ , jjjs++)
              {
                smi = iiis ;
                smj = jjjs ;
                alskys_f (li_p , &smi , &smj) ;
                *(*(mdiag_pp + smj) + smi) += *(em_p + (il++) * kfke + jl) ;
              }
            }   /*  Ende der Schleifen ueber transp. Nebendiagonaldatenbl.  */
          }     /*  Ende der Einspeicherung des transp. Nebendiag.-Datenbl. */
          
          fillin_f (10 , iy , jy) ;
          
        }       /*  Ende der EM-Spaltenschleife                             */

        /*  Einspeichern in Systembelastungsvektor:                         */

        iiis = (iy - 1) * kfmx ;

        for (j = 0 ; j < kf ; j++ , iiis++)
        {
          lls = (*(li_p + iiis) > iiis) ? *(li_p + iiis) : iiis ;
          *(b_p + lls) += *(eb_p + ik*kf + j) ;

        }     /*  ... Ende des Einspeicherns in Belastungsvektor            */

        fillin_f (20 , iy , jy) ;

      }       /*  ... Ende des Einspeichern eines Elements                  */
      
    }         /*  ... Ende der Schleife ueber alle Elemente                 */

    fillin_f (1000 , 0L , 0L) ;
      
    freesp_f   (em_p) ;
    freesp_f   (eb_p) ;
    return 1      ;
}


/*  Ueberpruefen, ob der Vektor li_p (identische Verschiebungen)
    Alias-Verweise auf andere (hoehere) Freiheitsgrade fuer die
    Freiheitsgrade is und js enthaelt, zurueckgeliefert wird in
    jedem Fall die Position in der Systemsteifigkeitsmatrix oberhalb
    der Hauptdiagonalen, entweder fuer die Original-Freiheitsgrade
    oder unter Beruecksichtigung von "Aliassen":                         */

void alskys_f (long *li_p , long *is , long *js)
{
   long k ;

   *is = (*(li_p + *is) > *is) ? *(li_p + *is) : *is ;
   *js = (*(li_p + *js) > *js) ? *(li_p + *js) : *js ;

    /*  ... und is und js sind die tatsaechlich zu verwendenden
            Ziele fuer das Einspeichern                                     */

   if (*is > *js)
    {                         /*  ... wuerde Ziel unter H.-Diag. liegen     */
      k   = *is ;
      *is = *js ;
      *js = k   ;             /*  ... es wurde getauscht                    */
    }
    
   *is = *js - *is ;  /* ... und abgeliefert wird die Spalte js und die 
                             "Hoehe ueber der Hauptdiagonalen is"           */
}
