/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Solver, der den FEM-Algorithmus startet                                  */
/*  =======================================                                  */
/*                                                                           */
/*  Vor dem Starten des FEM-Algorithmus (Funktionen femalg_f bzw. feskal_f)  */
/*  wird die Funktion preleq_f aufgerufen, die die Vorbereitungen fuer die   */
/*  Ablaufverfolgung (Struktur des Gleichungssystems, Protokollausschriften) */
/*  erledigt.                                                                */
/*                                                                           */
/*  Eingabe:  FemModTask  == FMT_CALCULATEDIS_f  --->  Klassisches Band-     */
/*                                                     matrixverfahren       */
/*                        != FMT_CALCULATEDIS_f  --->  "Skyline-Verfahren"   */
/*                                                                           */
/*  Return-Wert:   1  --->  Erfolg, Verschiebungen wurden berechnet,         */
/*                 0  --->  Misserfolg                                       */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    "femset_f.h"

int solver_f (int FemModTask)
{
   preleq_f () ;

   if (FemModTask == FMT_CALCULATEDIS_f)
   { 
     /* Klassisches Bandmatrix-Verfahren:                                    */
   
     return femalg_f (kx_f   , kf_f   , ke_f   , kp_f   , ne_f   , nk_f ,
                      xy_p_f , km_p_f , et_p_f , ep_p_f , kr_p_f ,
                      bk_p_f , sc_p_f , la_p_f , aa_p_f , li_p_f , 
                      pu_p_f , uv_p_f) ;
   }
   else                   
   { 
     /* ... und wenn nicht ausdruecklich ein anderes Loesungsverfahren 
            angefordert wurde, wird das "Skyline-Verfahren" verwendet:       */
            
     return feskal_f (kx_f   , kf_f   , ke_f   , kp_f   , ne_f   , nk_f ,
                      xy_p_f , km_p_f , et_p_f , ep_p_f , kr_p_f ,
                      bk_p_f , sc_p_f , la_p_f , aa_p_f , li_p_f , 
                      pu_p_f , uv_p_f) ;
   }
}
