/* *********************************************************** */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                 */
/*                                                             */
/*  Ruecksortierung nach Loesung des inversen                  */
/*  Eigenwertproblems                                          */
/*  =========================================                  */
/*                                                             */
/*  Die Spalten der Matrix X (m Zeilen, neiwe Spalten)         */
/*  werden in umgekehrte Reihenfolge gebracht, ebenso die      */
/*  Elemente des Vektors, der ab eiwe_p gespeichert ist, die   */
/*  zusaetzlich in ihre reziproken Werte geaendert werden      */
/*                                                             */
/*  Eingabe:   x_p    -  Matrix X mit m*neiwe Elementen        */
/*             eiwe_p -  Vektor der Eigenwerte mit neiwe       */
/*                       Elemente                              */
/*             m      -  Zeilenanzahl der Matrix X             */
/*             neiwe  -  Spaltenanzahl der Matrix X            */
/*                                                             */
/*  Ausgabe   *x_p    - Matrix X (umsortiert)                  */
/*            *eiwe_p - Vektor der Eigenwerte (umsortiert)     */
/*                                                             */
/*  Autor: J. Dankert                                          */
/* *********************************************************** */

#include  <math.h>
#include    "femset_f.h"

void sortiv_f (double *x_p , double *eiwe_p , long m , long neiwe)
{
  long    i  , i1 , i2 , j1 , j2 ;
  double  xx ;

  for (i = 0 ; i < neiwe ; i++)
	  *(eiwe_p + i) = 1./(*(eiwe_p + i)) ;

  j1 = 0 ;
  j2 = neiwe - 1 ;

  while (j1 < j2) 
  {
      xx             = *(eiwe_p + j1) ;
	  *(eiwe_p + j1) = *(eiwe_p + j2) ;
	  *(eiwe_p + j2) = xx             ;

      for (i = 0 , i1 = j1 , i2 = j2 ; i < m ; i++ , i1 += neiwe , i2 += neiwe)
	  {
		  xx          = *(x_p + i1) ; 
	      *(x_p + i1) = *(x_p + i2) ;
		  *(x_p + i2) = xx          ;
	  }

	  j1++ ;
	  j2-- ;
  }
}
