/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Aufbau der Elementsteifigkeitsbeziehung 'SV16'                           */
/*  ==============================================                           */
/*                                                                           */
/*  Die uebergebene Elementnummer ie ist die Nummer der "Benutzer-           */
/*  Numerierung" (1 ... ne) der Elemente, in km_p stehen ebenfalls die       */
/*  Knotennummern der "Benutzer-Numerierung" (1 ... nk).                     */
/*                                                                           */
/*  Die Felder sind allerdings (wie in C ueblich) mit der unteren Index-     */
/*  grenze 0 definiert, auf km_p [3] [0] steht also z. B. die Knotennummer   */
/*  des ersten zum Element 4 gehoerenden Elementknotens (km_p ist mit ke     */
/*  Spalten vereinbart, was einen Zugriff auf dasselbe Element ueber         */
/*  *(km_p + kemx*3) zulaesst.                                               */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    <math.h>
#include    "femset_f.h"

void gaussi (double *xi_p , double *eta_p , double *w_p)
{
  double alpha , w1 , w2 , w6 ;

  alpha   = sqrt (.6) ;
  w1      = 64./81.   ;
  w2      = 40./81.   ;
  w6      = 25./81.   ;

  *(xi_p  ) =   0.    ;
  *(xi_p+1) =   0.    ;
  *(xi_p+2) =   alpha ;
  *(xi_p+3) =   0.    ;
  *(xi_p+4) = - alpha ;
  *(xi_p+5) =   alpha ;
  *(xi_p+6) =   alpha ;
  *(xi_p+7) = - alpha ;
  *(xi_p+8) = - alpha ;

  *(eta_p  ) =   0.    ;
  *(eta_p+1) =   alpha ;
  *(eta_p+2) =   0.    ;
  *(eta_p+3) = - alpha ;
  *(eta_p+4) =   0.    ;
  *(eta_p+5) =   alpha ;
  *(eta_p+6) = - alpha ;
  *(eta_p+7) = - alpha ;
  *(eta_p+8) =   alpha ;

  *(w_p  ) = w1 ;
  *(w_p+1) = w2 ;
  *(w_p+2) = w2 ;
  *(w_p+3) = w2 ;
  *(w_p+4) = w2 ;
  *(w_p+5) = w6 ;
  *(w_p+6) = w6 ;
  *(w_p+7) = w6 ;
  *(w_p+8) = w6 ;
}

void gderiv (double xi , double eta , double *dgnxi_p , double *dgneta_p)
{
  *(dgnxi_p  ) = (1. + eta) * (2. * xi + eta) * .25 ;
  *(dgnxi_p+1) = (1. - eta) * (2. * xi - eta) * .25 ;
  *(dgnxi_p+2) = (1. + eta) * (2. * xi - eta) * .25 ;
  *(dgnxi_p+3) = (1. - eta) * (2. * xi + eta) * .25 ;
  *(dgnxi_p+4) =   (1. - eta * eta) * .5 ;
  *(dgnxi_p+5) = - (1. - eta * eta) * .5 ;
  *(dgnxi_p+6) = - (1. + eta) * xi ;
  *(dgnxi_p+7) = - (1. - eta) * xi ;

  *(dgneta_p  ) = (1. + xi) * (2. * eta + xi) * .25 ;
  *(dgneta_p+1) = (1. + xi) * (2. * eta - xi) * .25 ;
  *(dgneta_p+2) = (1. - xi) * (2. * eta - xi) * .25 ;
  *(dgneta_p+3) = (1. - xi) * (2. * eta + xi) * .25 ;
  *(dgneta_p+4) = - (1. + xi) * eta ;
  *(dgneta_p+5) = - (1. - xi) * eta ;
  *(dgneta_p+6) =   (1. - xi * xi) * .5 ;
  *(dgneta_p+7) = - (1. - xi * xi) * .5 ;
}

int sv16_f (long ie   , double *xy_p , long   *km_p , double *ep_p ,
            long kemx , long    kx   , long   *kf   , long   *ke   ,
            long kp   , double *em_p , double *eb_p)
{
  long     i , j , kii , ig , ki [8] ;     
  double   w[9] , xi [9] , eta [9] , x [8] , y [8] , h [9] , hh [9] ;
  double   dgnxi [9] , dgneta [9] , b [12] ;
  double   gqs [64] , dg [48] , hdg [48] , dgt [48] , emh [256] ;
  double   t , e , ny , fak , xiig , etaig , j11 , j12 , j21 , j22 , detj ;

  gaussi (xi , eta , w) ;

  *ke = 8 ;                                 /*  Knoten pro Element           */
  *kf = 2 ;                                 /*  Freiheitsgrade pro Knoten    */

  for (i = 0 ; i < 8 ; i++)
  {
    kii = *(km_p + (ie-1)*kemx + i) ;
	*(x  + i) = *(xy_p + (kii-1)*kx) ;      /*  Knotenkoordinaten            */
	*(y  + i) = *(xy_p + (kii-1)*kx + 1) ;
	*(ki + i) = kii ;                       /*  Knotennummern des Elements   */
  }

  t   = *(ep_p + (ie-1)*kp)     ;           /*  Elementparameter             */
  e   = *(ep_p + (ie-1)*kp + 1) ;
  ny  = *(ep_p + (ie-1)*kp + 2) ;
  fak = e / (1-ny*ny) ;

  *(h  ) = fak ;
  *(h+1) = fak * ny ;
  *(h+2) = 0. ;
  *(h+3) = fak * ny ;
  *(h+4) = fak ;
  *(h+5) = 0. ;
  *(h+6) = 0. ;
  *(h+7) = 0. ;
  *(h+8) = fak * (1. - ny) * .5 ;

  for (i = 0 ; i < 256 ; i++)
     *(em_p + i) = 0. ;

  for (ig = 0 ; ig < 9 ; ig++)
  {
	  xiig  = *(xi +ig) ;
	  etaig = *(eta+ig) ;
	  gderiv (xiig , etaig , dgnxi , dgneta) ;

      j11 = 0. ;
      j12 = 0. ;
      j21 = 0. ;
      j22 = 0. ;

      for (j = 0 ; j < 8 ; j++)
	  {
        j11 = j11 + *(dgnxi+j)  * *(x+j) ;
        j12 = j12 + *(dgnxi+j)  * *(y+j) ;
        j21 = j21 + *(dgneta+j) * *(x+j) ;
        j22 = j22 + *(dgneta+j) * *(y+j) ;
      }

      detj = j11 * j22 - j12 * j21 ;

      *(b   ) =   j22 / detj ;
      *(b+ 1) = - j12 / detj ;
      *(b+ 2) =   0. ;
      *(b+ 3) =   0. ;
      *(b+ 4) =   0. ;
      *(b+ 5) =   0. ;
      *(b+ 6) = - j21 / detj ;
      *(b+ 7) =   j11 / detj ;
      *(b+ 8) = - j21 / detj ;
      *(b+ 9) =   j11 / detj ;
      *(b+10) =   j22 / detj ;
      *(b+11) = - j12 / detj ;

      for (i = 0 ; i < 64 ; i++)
         *(gqs + i) = 0. ;

      for (j = 0 ; j < 8 ; j++)
	  {
          *(gqs + j*2     ) = *(dgnxi+j)  ; 
		  *(gqs + j*2 + 16) = *(dgneta+j) ;
		  *(gqs + j*2 + 33) = *(dgnxi+j)  ; 
		  *(gqs + j*2 + 49) = *(dgneta+j) ;
	  }

	  matmul_f (b , gqs , dg , 3 , 4 , 16) ;

      for (i = 0 ; i < 3 ; i++)
         for (j = 0 ; j < 3 ; j++)
			 *(hh + i*3 + j) = *(h + i*3 + j) * t * detj * *(w + ig) ;

	  matmul_f (hh , dg , hdg , 3 , 3 , 16) ;

      for (i = 0 ; i < 3 ; i++)
         for (j = 0 ; j < 16 ; j++)
			 *(dgt + j*3 + i) = *(dg + i*16 + j) ;

	  matmul_f (dgt , hdg , emh , 16 , 3 , 16) ;

      for (i = 0 ; i < 256 ; i++)
		  *(em_p + i) += *(emh + i) ;
  }

  return 1 ;
}
