/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Aufbau des Matrizen-Eigenwertproblems fuer Eigenschwingungen             */
/*  ============================================================             */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    "femset_f.h"
#include    "w_fs_res.h"

long ekmpos_f (long is , long js , long ib) ;

int syseig_f (double *a_p  , double *b_p  , long  n    , long    ib  ,
              long    ne   , long    nk   ,
              long    kx   , long    kfmx , long  kemx , long    kp  ,
              double *xy_p , long   *km_p , long *et_p , double *ep_p)
{
    double   *ek_p , *em_p ;                /*  Elementsteifigkeitsbeziehung */

    long      i   , j    , ie   , ik  , jk    , iy   , jy  , kf , ke  ,
              me  , it   , jt   , il  , jl    , suc  , iis ,
              jjs , iiis , jjjs , lls , isave , kfke ;

    me = kfmx * kemx ;         /* Maximale Anzahl der Elementfreiheitsgrade  */

    /***********  Speicherplatz fuer                        ******************/
    /***********  Elementsteifigkeitsbeziehung allokieren:  ******************/

    suc = ((ek_p = (double *) calloc ((int) (me * me) ,
                                      sizeof (double))) != NULL) ;
    if (suc) suc = ((em_p = (double *) calloc ((int) me * me ,
                                               sizeof (double))) != NULL) ;
    if (! suc) 
    {
      infbox_f (S_SORRY_f , S_NOELEMSPACE_f) ;
      return 0 ;
    }

    darnul_f (a_p , n * ib) ;
    fillin_f (0 , 0L , 0L) ;

    for (ie = 0 ; ie < ne ; ie++)       /*  Elementnummern  ie = 0 ... ne-1  */
    {
      darnul_f (ek_p , me * me) ;       /*  Nullsetzen der                   */
      darnul_f (em_p , me * me) ;       /*  Elementmatrizen                  */

      ke = kemx ;        /*  Es werden Maximalwerte vermutet, die elemat_f   */
      kf = kfmx ;        /*  modifizieren kann                               */

      elemat_f (ie + 1 , xy_p , km_p , et_p , ep_p , kemx ,
                kx     , &kf  , &ke  , kp   , ek_p , em_p) ;

      kfke = kf * ke ;

      /*  Einspeichern in die Systemmatrizen:                                */

      for (ik = 0 ; ik < ke ; ik++)           /*  Elementknoten werden       */
      {                                       /*  von  0 ... ke-1  gezaehlt  */

        iy = *(km_p + ie*kemx + ik) ;         /*  ... sind die EXTERNEN ...  */

        for (jk = ik ; jk < ke ; jk++)
        {
          jy = *(km_p + ie*kemx + jk) ;       /*  (1 ... nk) KNOTENNUMMERN   */

          iis = (iy - 1) * kfmx     ;         /*  Freiheitsgrad-POSITIONEN   */
          jjs = (jy - 1) * kfmx     ;         /*  des Ziel-Leitelements      */

          it = ik * kf              ;         /*  POSITIONEN des Leit-       */
          jt = jk * kf              ;         /*  elements in EK und EM      */

          if (jy == iy)                       /*  Hauptdiagonaldatenblock    */
          {                                   /*  =======================    */
            il   = it  ;
            iiis = iis ;
            for (i = 0 ; i < kf ; i++ , il++ , iiis++ , jjs++)
            {
              for (j = i , jl = il , jjjs = jjs ; j < kf ; j++ , jjjs++)
              {
                lls = ekmpos_f (iiis , jjjs , ib) ;
                *(a_p + lls) += *(ek_p + il*kfke + jl  ) ;
                *(b_p + lls) += *(em_p + il*kfke + jl++) ;
              }
            }    /*  Ende der Schleifen ueber Hauptdiagonaldatenblock        */
          }      /*  Ende der Einspeicherung eines Hauptdiagonaldatenblocks  */

          else if (jy > iy)                   /*  Nebendiagonaldatenblock    */
          {                                   /*  =======================    */
            il   = it  ;
            iiis = iis ;
            for (i = 0 ; i < kf ; i++ , il++ , iiis++)
            {
              for (j = 0 , jl = jt , jjjs = jjs ; j < kf ; j++ , jjjs++)
              {
                lls = ekmpos_f (iiis , jjjs , ib) ;
                *(a_p + lls) += *(ek_p + il*kfke + jl  ) ;
                *(b_p + lls) += *(em_p + il*kfke + jl++) ;
              }
            }   /*  Ende der Schleifen ueber Nebendiagonaldatenblock         */
          }     /*  Ende der Einspeicherung eines Nebendiagonaldatenblocks   */

          else                                /*  Transp. Nebendiag.-Block   */
          {                                   /*  ========================   */
            jl    = jt    ;
            isave = iis   ;                   /*  Zeilen- und Spaltennum-    */
            iis   = jjs   ;                   /*  mern fuer SM werden        */
            jjs   = isave ;                   /*  werden getauscht           */
            iiis  = iis   ;
            for (i = 0 ; i < kf ; i++ , jl++ , iiis++)
            {
              for (j = 0 , il = it , jjjs = jjs ; j < kf ; j++ , jjjs++)
              {
                lls = ekmpos_f (iiis , jjjs , ib)   ;

				*(a_p + lls) += *(ek_p + (il  )*kfke + jl) ;
				*(b_p + lls) += *(em_p + (il++)*kfke + jl) ;
              }
            }   /*  Ende der Schleifen ueber transp. Nebendiagonaldatenbl.   */
          }     /*  Ende der Einspeicherung des transp. Nebendiag.-Datenbl.  */
          
          fillin_f (10 , iy , jy) ;
          
        }       /*  Ende der EK-EM-Spaltenschleife                           */
      }         /*  ... Ende des Einspeichern eines Elements                 */
    }           /*  ... Ende der Schleife ueber alle Elemente                */

    fillin_f (1000 , 0L , 0L) ;
      
    freesp_f   (ek_p) ;
    freesp_f   (em_p) ;
    return 1      ;
}

long ekmpos_f (long is , long js , long ib)
{
  long  ilin , jlin , isave ;

  ilin = is ;
  jlin = js ;

  if (ilin > jlin) 
  {                            /*  ... wuerde Ziel unter H.-Diag. liegen     */
    isave = ilin  ;
    ilin  = jlin  ;
    jlin  = isave ;            /*  ... es wurde getauscht                    */
  }

  return (ilin * ib + jlin - ilin) ;                     /*  EK-EM-Position  */
}
