/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*  Aufbau der Elementsteifigkeitsbeziehung 'Torsd6'                         */
/*  ================================================                         */
/*                                                                           */
/*  Die uebergebene Elementnummer ie ist die Nummer der "Benutzer-           */
/*  Numerierung" (1 ... ne) der Elemente, in km_p stehen ebenfalls die       */
/*  Knotennummern der "Benutzer-Numerierung" (1 ... nk).                     */
/*                                                                           */
/*  Die Felder sind allerdings (wie in C ueblich) mit der unteren Index-     */
/*  grenze 0 definiert, auf km_p [3] [0] steht also z. B. die Knotennummer   */
/*  des ersten zum Element 4 gehoerenden Elementknotens (km_p ist mit ke     */
/*  Spalten vereinbart, was einen Zugriff auf dasselbe Element ueber         */
/*  *(km_p + kemx*3) zulaesst.                                               */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    <math.h>

int torsd6_f (long ie   , double *xy_p , long   *km_p , double *ep_p ,
              long kemx , long    kx   , long   *kf   , long   *ke   ,
              long kp   , double *em_p , double *eb_p)
{
  long     k1 , k2 , k3 ;
  double   x1 , y1 , x2 , y2 , x3 , y3 , x21 , x31 , x32 , y21 , y31 , y32 ,
	       h11 , h12 , h22 , h1s , h2s , A2 , fak ;

  *ke = 6 ;                        /*  Knoten pro Element                    */
  *kf = 1 ;                        /*  Freiheitsgrade pro Knoten             */

  k1 = *(km_p + (ie-1)*kemx)     ; /*  Knotennummern des Elements            */
  k2 = *(km_p + (ie-1)*kemx + 1) ;
  k3 = *(km_p + (ie-1)*kemx + 2) ;

  x1 = *(xy_p + (k1-1)*kx)     ;   /*  Knotenkoordinaten                     */
  y1 = *(xy_p + (k1-1)*kx + 1) ;
  x2 = *(xy_p + (k2-1)*kx)     ;   
  y2 = *(xy_p + (k2-1)*kx + 1) ;
  x3 = *(xy_p + (k3-1)*kx)     ;   
  y3 = *(xy_p + (k3-1)*kx + 1) ;

  x21 = x2 - x1 ;                  /*  Elementabmessungen                    */
  x31 = x3 - x1 ;
  x32 = x3 - x2 ;
  y21 = y2 - y1 ;
  y31 = y3 - y1 ;
  y32 = y3 - y2 ;

  h11 = x32*x32 + y32*y32 ;
  h12 = -x31*x32 - y31*y32 ;
  h22 = x31*x31 + y31*y31  ;
  h1s = h11 + h12 ;
  h2s = h12 + h22 ;
  A2  = x21*y31 - x31*y21 ;

  if (fabs(A2) < 1.e-8) return 0 ;
  
  fak = 1. / A2 ;

  *(em_p     ) = h11 * 0.5 * fak ;
  *(em_p +  1) = - h12/6 * fak ;
  *(em_p +  2) = h1s/6 * fak ;
  *(em_p +  3) = h12 * 2/3 * fak ;
  *(em_p +  4) = 0 ;
  *(em_p +  5) = - h1s * 2/3 * fak ;
  *(em_p +  7) = h22 * 0.5 * fak ;
  *(em_p +  8) = h2s/6 * fak ;
  *(em_p +  9) = h12 * 2/3 * fak ;
  *(em_p + 10) = - h2s *2/3 * fak ;
  *(em_p + 11) = 0 ;
  *(em_p + 14) = (h1s + h2s) * 0.5 * fak ;
  *(em_p + 15) = 0 ;
  *(em_p + 16) = -h2s * 2/3 * fak ;
  *(em_p + 17) = -h1s * 2/3 * fak ;
  *(em_p + 21) = (h1s + h22) * 4/3 * fak ;
  *(em_p + 22) = - h1s * 4/3 * fak ;
  *(em_p + 23) = - h2s * 4/3 * fak ;
  *(em_p + 28) = (h1s + h22) * 4/3 * fak ;
  *(em_p + 29) = h12 * 4/3 * fak ;
  *(em_p + 35) = (h1s + h22) * 4/3 * fak ;

  *(eb_p     ) = 0 ;
  *(eb_p +  1) = 0 ;
  *(eb_p +  2) = 0 ;
  *(eb_p +  3) = - A2/6 ;
  *(eb_p +  4) = - A2/6 ;
  *(eb_p +  5) = - A2/6 ;

  return 1 ;
}
