/* ************************************************************************* */
/*  ###  Finite-Elemente-Baukasten FEMSET  ###                               */
/*                                                                           */
/*                           T                                               */
/*  Berechnung des Produkts y * B * x mit den Vektoren y und x und der       */
/*  symmetrischen Bandmatrix B (m Zeilen bzw. Spalten, Bandweite ib)         */
/*  ==================================================================       */
/*                                                                           */
/*  B ist eine symmetrische Bandmatrix mit m Zeilen und der Bandweite ib,    */
/*  von der die wesentlichen m*ib Elemente in einem eindimensionalen Feld    */
/*  zeilenweise dicht gespeichert sind.                                      */
/*                                                                           */
/*  y und x sind Vektoren mit jeweils m Elementen. Sie sind die ispy-te      */
/*  Spalte der Matrix, die ab Position xy_p gespeichert ist, bzw. deren      */
/*  ispx-te Spalte. Die Zaehlung der Spalten beginnt jeweils bei 0.          */
/*  Die Matrix xy_p hat m Zeilen und neiwe Spalten.                          */
/*                                                                           */
/*                                           T                               */
/*  Return-Wert ist die double-Variable r = y * B * x.                       */
/*                                                                           */
/*  Autor: J. Dankert                                                        */
/* ************************************************************************* */

#include    "femset_f.h"

double ytbibx_f (double *xy_p , double *b_p , long m   , long ib ,
				 long   neiwe , long ispy   , long ispx)
{
  long      i , j , jj , jxu , jxo , jjx , ibw1  , idia = 1 , ib12 ;
  double    r = 0. , s ;

  ibw1 = ib - 1 ;
  ib12 = ibw1 * ibw1 ;

  for (i = 1 ; i <= m ; i++)
  {
	  if (i < ib)
	  {
		  jj  = i ;
		  jxu = 1 ;
	  }
	  else
	  {
		  jj  = idia - ib12 ;
		  jxu = i    - ibw1 ;
	  }

	  jxo = i + ibw1 ;
	  if (jxo > m) jxo = m ;

	  /* Schleife ueber alle Elemente einschliesslich Hauptdiagonalelement: */
	  s = 0. ;
      for (j = jxu , jjx = (jxu-1)*neiwe + ispx ; j <= i ; j++ , jjx += neiwe)
	  {
		  s  += *(b_p + jj-1) * *(xy_p + jjx) ;
		  jj += ibw1 ;
	  } /* end j */

	  if ((i != m) | (ib != 1))
	  {
		  /* Schleife ueber die uebrigen Elemente der Zeile i: */
          jxu = i    + 1 ;
		  jj  = idia + 1 ;
          for (j = jxu , jjx = (jxu-1)*neiwe + ispx ; j <= jxo ; j++ , jjx += neiwe)
		  {
		      s  += *(b_p + jj-1) * *(xy_p + jjx) ;
		      jj++ ;
		  } /* end j */
	  } /* end if */

	  r    += *(xy_p + (i-1)*neiwe + ispy) * s ;
	  idia += ib ;

  } /* end i */

  return (r) ;
}
